@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  cache.c - contains the cache routines                                    */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,16 +18,99 @@
 /*****************************************************************************/
 
 #include "gftp.h"
-static const char cvsid[] = "$Id: cache.c,v 1.6 2002/11/27 14:29:55 masneyb Exp $";
+static const char cvsid[] = "$Id: cache.c,v 1.17 2003/08/12 00:04:37 masneyb Exp $";
 
-char *
-gftp_cache_get_url_prefix (gftp_request * request)
+struct gftp_cache_entry_tag
 {
-  if (strcmp (request->protocol_name, "HTTP") == 0 &&
-      strcmp (request->proxy_config, "ftp") == 0)
-    return ("ftp");
+  char *url,
+       *file;
+  int server_type;
+  time_t expiration_date;
 
-  return (request->url_prefix);
+  char *pos1,
+       *pos2,
+       *pos3;
+};
+  
+typedef struct gftp_cache_entry_tag gftp_cache_entry;
+
+
+static int
+gftp_parse_cache_line (gftp_request * request, gftp_cache_entry * centry, 
+                       char *line)
+{
+  char *pos;
+
+  memset (centry, 0, sizeof (*centry));
+
+  if ((pos = strchr (line, '\t')) == NULL || *(pos + 1) == '\0')
+    {
+      if (request != NULL)
+        request->logging_function (gftp_logging_error, request,
+                            _("Error: Invalid line %s in cache index file\n"), 
+                            line);
+      return (-1);
+    }
+
+  centry->pos1 = pos;
+  *pos++ = '\0';
+  centry->url = line;
+  centry->file = pos;
+  
+  if ((pos = strchr (pos, '\t')) == NULL || *(pos + 1) == '\0')
+    {
+      if (request != NULL)
+        request->logging_function (gftp_logging_error, request,
+                            _("Error: Invalid line %s in cache index file\n"), 
+                            line);
+      return (-1);
+    }
+
+  centry->pos2 = pos;
+  *pos++ = '\0';
+  centry->server_type = strtol (pos, NULL, 10);
+
+  if ((pos = strchr (pos, '\t')) == NULL || *(pos + 1) == '\0')
+    {
+      if (request != NULL)
+        request->logging_function (gftp_logging_error, request,
+                            _("Error: Invalid line %s in cache index file\n"), 
+                            line);
+      return (-1);
+    }
+
+  centry->pos3 = pos;
+  *pos++ = '\0';
+  centry->expiration_date = strtol (pos, NULL, 10);
+
+  return (0);
+}
+
+
+static void
+gftp_restore_cache_line (gftp_cache_entry * centry, char *line)
+{
+  if (centry->pos1 != NULL)
+    *centry->pos1 = '\t';
+
+  if (centry->pos2 != NULL)
+    *centry->pos2 = '\t';
+
+  if (centry->pos3 != NULL)
+    *centry->pos3 = '\t';
+}
+
+
+void
+gftp_generate_cache_description (gftp_request * request, char *description,
+                                 size_t len, int ignore_directory)
+{
+  g_snprintf (description, len, "%s://%s@%s:%d%s",
+              request->url_prefix,
+              request->username == NULL ? "" : request->username,
+              request->hostname == NULL ? "" : request->hostname,
+              request->port, 
+              ignore_directory || request->directory == NULL ? "" : request->directory);
 }
 
 
@@ -35,8 +118,13 @@ int
 gftp_new_cache_entry (gftp_request * request)
 {
   char *cachedir, *tempstr, *temp1str;
-  int cache_fd, fd;
+  int cache_fd, fd, cache_ttl;
   ssize_t ret;
+  time_t t;
+
+  gftp_lookup_request_option (request, "cache_ttl", &cache_ttl);
+  time (&t);
+  t += cache_ttl;
 
   cachedir = expand_path (BASE_CONF_DIR "/cache");
   if (access (cachedir, F_OK) == -1)
@@ -44,7 +132,7 @@ gftp_new_cache_entry (gftp_request * request)
       if (mkdir (cachedir, S_IRUSR | S_IWUSR | S_IXUSR) < 0)
         {
           if (request != NULL)
-            request->logging_function (gftp_logging_error, request->user_data,
+            request->logging_function (gftp_logging_error, request,
                                  _("Error: Could not make directory %s: %s\n"),
                                  cachedir, g_strerror (errno));
 
@@ -53,18 +141,14 @@ gftp_new_cache_entry (gftp_request * request)
     }
 
   tempstr = g_strdup_printf ("%s/index.db", cachedir);
-  if ((fd = open (tempstr, O_WRONLY | O_APPEND | O_CREAT, 
-                  S_IRUSR | S_IWUSR)) == -1)
+  if ((fd = gftp_fd_open (request, tempstr, O_WRONLY | O_APPEND | O_CREAT, 
+                          S_IRUSR | S_IWUSR)) == -1)
     {
-      if (request != NULL)
-        request->logging_function (gftp_logging_error, request->user_data,
-                                   _("Error: Cannot open local file %s: %s\n"),
-                                   tempstr, g_strerror (errno));
-
       g_free (tempstr);
       g_free (cachedir);
       return (-1);
     }
+
   g_free (tempstr);
 
   tempstr = g_strdup_printf ("%s/cache.XXXXXX", cachedir);
@@ -72,7 +156,7 @@ gftp_new_cache_entry (gftp_request * request)
     {
       g_free (tempstr);
       if (request != NULL)
-        request->logging_function (gftp_logging_error, request->user_data,
+        request->logging_function (gftp_logging_error, request,
                                  _("Error: Cannot create temporary file: %s\n"),
                                  g_strerror (errno));
       return (-1);
@@ -80,21 +164,21 @@ gftp_new_cache_entry (gftp_request * request)
   g_free (cachedir);
 
   lseek (fd, 0, SEEK_END);
-  temp1str = g_strdup_printf ("%s://%s@%s:%d%s\t%s\n", 
-                           gftp_cache_get_url_prefix (request),
+  temp1str = g_strdup_printf ("%s://%s@%s:%d%s\t%s\t%d\t%ld\n", 
+                           request->url_prefix,
                            request->username == NULL ? "" : request->username,
                            request->hostname == NULL ? "" : request->hostname,
                            request->port, 
                            request->directory == NULL ? "" : request->directory,
-                           tempstr);
+                           tempstr, request->server_type, t);
   g_free (tempstr);
-  ret = gftp_write (NULL, temp1str, strlen (temp1str), fd);
+  ret = gftp_fd_write (NULL, temp1str, strlen (temp1str), fd);
   g_free (temp1str);
 
   if (close (fd) != 0 || ret < 0)
     {
       if (request != NULL)
-        request->logging_function (gftp_logging_error, request->user_data,
+        request->logging_function (gftp_logging_error, request,
                                    _("Error closing file descriptor: %s\n"),
                                    g_strerror (errno));
 
@@ -109,20 +193,19 @@ gftp_new_cache_entry (gftp_request * request)
 int
 gftp_find_cache_entry (gftp_request * request)
 {
-  char *indexfile, *pos, buf[BUFSIZ], description[BUFSIZ];
+  char *indexfile, buf[BUFSIZ], description[BUFSIZ];
   gftp_getline_buffer * rbuf;
+  gftp_cache_entry centry;
   int indexfd, cachefd;
-  size_t len;
+  time_t now;
 
-  g_snprintf (description, sizeof (description), "%s://%s@%s:%d%s",
-              gftp_cache_get_url_prefix (request),
-              request->username == NULL ? "" : request->username,
-              request->hostname == NULL ? "" : request->hostname,
-              request->port, 
-              request->directory == NULL ? "" : request->directory);
+  time (&now);
+
+  gftp_generate_cache_description (request, description, sizeof (description),
+                                   0);
 
   indexfile = expand_path (BASE_CONF_DIR "/cache/index.db");
-  if ((indexfd = open (indexfile, O_RDONLY)) == -1)
+  if ((indexfd = gftp_fd_open (NULL, indexfile, O_RDONLY, 0)) == -1)
     {
       g_free (indexfile);
       return (-1);
@@ -132,37 +215,26 @@ gftp_find_cache_entry (gftp_request * request)
   rbuf = NULL;
   while (gftp_get_line (NULL, &rbuf, buf, sizeof (buf), indexfd) > 0)
     {
-      len = strlen (buf);
-
-      if (!((pos = strrchr (buf, '\t')) != NULL && *(pos + 1) != '\0'))
-	continue;
-
-      len = strlen (description);
-      if (pos - buf != len)
+      if (gftp_parse_cache_line (request, &centry, buf) < 0)
         continue;
 
-      if (strncmp (buf, description, len) == 0)
+      /* See if this entry is still valid... */
+      if (centry.expiration_date < now)
+        continue;
+
+      if (strcmp (description, centry.url) == 0)
 	{
-	  pos++;
 	  if (close (indexfd) != 0)
             {
               if (request != NULL)
-                request->logging_function (gftp_logging_error, 
-                                       request->user_data,
+                request->logging_function (gftp_logging_error, request,
                                        _("Error closing file descriptor: %s\n"),
                                        g_strerror (errno));
               return (-1);
             }
 
-	  if ((cachefd = open (pos, O_RDONLY)) == -1)
-            {
-              if (request != NULL)
-                request->logging_function (gftp_logging_error, 
-                                   request->user_data,
-                                   _("Error: Cannot open local file %s: %s\n"),
-                                   pos, g_strerror (errno));
-              return (-1);
-            }
+	  if ((cachefd = gftp_fd_open (request, centry.file, O_RDONLY, 0)) == -1)
+            return (-1);
 
           if (lseek (cachefd, 0, SEEK_END) == 0)
             { 
@@ -173,13 +245,13 @@ gftp_find_cache_entry (gftp_request * request)
           if (lseek (cachefd, 0, SEEK_SET) == -1)
             {
               if (request != NULL)
-                request->logging_function (gftp_logging_error, 
-                                       request->user_data,
+                request->logging_function (gftp_logging_error, request,
                                        _("Error: Cannot seek on file %s: %s\n"),
-                                       pos, g_strerror (errno));
+                                       centry.file, g_strerror (errno));
 
             }
 
+          request->server_type = centry.server_type;
 	  return (cachefd);
 	}
     }
@@ -191,13 +263,13 @@ gftp_find_cache_entry (gftp_request * request)
 void
 gftp_clear_cache_files (void)
 {
-  char *indexfile, buf[BUFSIZ], *pos;
+  char *indexfile, buf[BUFSIZ];
   gftp_getline_buffer * rbuf;
+  gftp_cache_entry centry;
   int indexfd;
-  size_t len;
 
   indexfile = expand_path (BASE_CONF_DIR "/cache/index.db");
-  if ((indexfd = open (indexfile, O_RDONLY)) == -1)
+  if ((indexfd = gftp_fd_open (NULL, indexfile, O_RDONLY, 0)) == -1)
     {
       g_free (indexfile);
       return;
@@ -206,11 +278,10 @@ gftp_clear_cache_files (void)
   rbuf = NULL;
   while (gftp_get_line (NULL, &rbuf, buf, sizeof (buf), indexfd) > 0)
     {
-      len = strlen (buf);
+      if (gftp_parse_cache_line (NULL, &centry, buf) < 0)
+        continue;
 
-      if (!((pos = strrchr (buf, '\t')) != NULL && *(pos + 1) != '\0'))
-	continue;
-      unlink (pos + 1);
+      unlink (centry.file);
     }
 
   close (indexfd);
@@ -220,77 +291,83 @@ gftp_clear_cache_files (void)
 
 
 void
-gftp_delete_cache_entry (gftp_request * request, int ignore_directory)
+gftp_delete_cache_entry (gftp_request * request, char *descr, 
+                         int ignore_directory)
 {
-  char *oldindexfile, *newindexfile, *pos, buf[BUFSIZ], description[BUFSIZ];
+  char *oldindexfile, *newindexfile, buf[BUFSIZ], description[BUFSIZ];
   gftp_getline_buffer * rbuf;
+  gftp_cache_entry centry;
   int indexfd, newfd;
-  size_t len, buflen;
+  time_t now;
   int remove;
+ 
+  g_return_if_fail (request != NULL || descr != NULL);
 
-  g_snprintf (description, sizeof (description), "%s://%s@%s:%d%s",
-              gftp_cache_get_url_prefix (request),
-              request->username == NULL ? "" : request->username,
-              request->hostname == NULL ? "" : request->hostname,
-              request->port, 
-              ignore_directory || request->directory == NULL ? "" : request->directory);
+  time (&now);
+  if (request != NULL)
+    {
+      gftp_generate_cache_description (request, description, sizeof (description),
+                                       ignore_directory);
+    }
+  else if (descr != NULL)
+    { 
+      strncpy (description, descr, sizeof (description));
+      description[sizeof (description) - 1] = '\0';
+    }
+  else
+    return;
 
   oldindexfile = expand_path (BASE_CONF_DIR "/cache/index.db");
-  if ((indexfd = open (oldindexfile, O_RDONLY)) == -1)
+  if ((indexfd = gftp_fd_open (NULL, oldindexfile, O_RDONLY, 0)) == -1)
     {
       g_free (oldindexfile);
       return;
     }
 
   newindexfile = expand_path (BASE_CONF_DIR "/cache/index.db.new");
-  if ((newfd = open (newindexfile, O_WRONLY | O_CREAT, 
-                     S_IRUSR | S_IWUSR)) == -1)
+  if ((newfd = gftp_fd_open (request, newindexfile, O_WRONLY | O_CREAT, 
+                             S_IRUSR | S_IWUSR)) == -1)
     {
-      if (request != NULL)
-        request->logging_function (gftp_logging_error, request->user_data,
-                                   _("Error: Cannot open local file %s: %s\n"),
-                                   newindexfile, g_strerror (errno));
-
       g_free (oldindexfile);
       g_free (newindexfile);
       return;
     }
 
   rbuf = NULL;
-  buflen = strlen (description);
-  while (gftp_get_line (NULL, &rbuf, buf, sizeof (buf), indexfd) > 0)
+  while (gftp_get_line (NULL, &rbuf, buf, sizeof (buf) - 1, indexfd) > 0)
     {
-      len = strlen (buf);
-
-      if (!((pos = strrchr (buf, '\t')) != NULL && *(pos + 1) != '\0'))
-        {
-          if (request != NULL)
-            request->logging_function (gftp_logging_error, request->user_data,
-                            _("Error: Invalid line %s in cache index file\n"), 
-                            buf);
-
-          continue;
-        }
+      if (gftp_parse_cache_line (request, &centry, buf) < 0)
+        continue;
 
       remove = 0;
-      if (ignore_directory)
+      if (centry.expiration_date < now)
+        remove = 1;
+      else if (ignore_directory)
         {
-          if (strncmp (buf, description, strlen (description)) == 0)
+          if (strncmp (centry.url, description, strlen (description)) == 0)
             remove = 1;
         }
       else
         {
-          if (buflen == pos - buf && strncmp (buf, description, pos - buf) == 0)
+          if (strcmp (centry.url, description) == 0)
             remove = 1;
         }
 
  
       if (remove)
-        unlink (pos + 1);
+        unlink (centry.file);
       else
         {
+          /* Make sure we put the tabs back in the line. I do it this way 
+             so that I don't have to allocate memory again for each line 
+             as we read it */
+          gftp_restore_cache_line (&centry, buf);
+
+          /* Make sure when we call gftp_get_line() that we pass the read size
+             as sizeof(buf) - 1 so that we'll have room to put the newline */
           buf[strlen (buf)] = '\n';
-          if (gftp_write (NULL, buf, strlen (buf), newfd) < 0)
+
+          if (gftp_fd_write (NULL, buf, strlen (buf), newfd) < 0)
             break;
         }
     }
