@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  gftp.h - include file for the whole ftp program                          */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -17,7 +17,7 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-/* $Id: gftp.h,v 1.16 2002/12/04 02:35:44 masneyb Exp $ */
+/* $Id: gftp.h,v 1.58 2003/08/21 20:24:08 masneyb Exp $ */
 
 #ifndef __GFTP_H
 #define __GFTP_H
@@ -35,7 +35,6 @@
 #endif
 #include <sys/wait.h>
 #include <sys/utsname.h>
-#include <sys/sysmacros.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <glib.h>
@@ -64,6 +63,14 @@
 #include <strings.h>
 #endif
 
+#ifdef USE_SSL
+#include <openssl/bio.h>
+#include <openssl/err.h>
+#include <openssl/rand.h>
+#include <openssl/ssl.h>
+#include <openssl/x509v3.h>
+#endif
+
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #include <locale.h>
@@ -77,6 +84,13 @@
 #define AF_LOCAL AF_UNIX
 #endif
 
+#ifdef HAVE_GETADDRINFO
+#define HAVE_IPV6
+#define GFTP_GET_AI_FAMILY(request)	(request != NULL && request->hostp != NULL ? request->hostp->ai_family : -1)
+#else
+#define GFTP_GET_AI_FAMILY(request)	AF_INET
+#endif
+
 /* We need the major() and minor() macros in the user interface. If they aren't
    defined by the system, we'll just define them here. */
 #ifndef major
@@ -89,10 +103,45 @@
 #define minor(dev) ((dev) & 0xff)
 #endif
 
-#ifdef HAVE_DMALLOC
+#ifdef WITH_DMALLOC
+
+#undef g_malloc
+#define g_malloc(size) \
+  _malloc_leap(__FILE__, __LINE__, size)
+
+#undef g_malloc0
+#define g_malloc0(size) \
+  _calloc_leap(__FILE__, __LINE__, 1, size)
+
+#undef g_realloc
+#define g_realloc(ptr, size) \
+  _realloc_leap(__FILE__, __LINE__, ptr, size)
+
+#undef g_strdup
+#define g_strdup(str) \
+  _strdup_leap(__FILE__, __LINE__, str)
+
+#undef g_free
+#define g_free(ptr) \
+  _free_leap(__FILE__, __LINE__, ptr)
+
 #include <dmalloc.h>
+
 #endif
 
+/* Server types (used by FTP protocol from SYST command) */
+#define GFTP_DIRTYPE_UNIX	1
+#define GFTP_DIRTYPE_EPLF	2
+#define GFTP_DIRTYPE_CRAY	3	
+#define GFTP_DIRTYPE_NOVELL	4	
+#define GFTP_DIRTYPE_DOS	5	
+#define GFTP_DIRTYPE_VMS	6	
+#define GFTP_DIRTYPE_OTHER 	7
+
+/* Error types */
+#define GFTP_ERETRYABLE		-1
+#define GFTP_EFATAL		-2
+
 /* Some general settings */
 #define BASE_CONF_DIR		"~/.gftp"
 #define CONFIG_FILE		BASE_CONF_DIR "/gftprc"
@@ -115,11 +164,6 @@ typedef enum gftp_logging_level_tag
   gftp_logging_misc
 } gftp_logging_level;
 
-typedef void (*gftp_logging_func)		( gftp_logging_level level, 
-						  void *ptr, 
-						  const char *string, ... );
-
-
 typedef struct gftp_file_tag gftp_file;
 
 #define GFTP_TRANS_ACTION_OVERWRITE		1
@@ -139,22 +183,24 @@ typedef struct gftp_file_tag gftp_file;
 struct gftp_file_tag 
 {
   char *file,			/* Our filename */
+       *utf8_file,		/* UTF-8 encoded filename for display purposes
+				   only. This is only set if file is not in 
+				   UTF-8 */
        *user,			/* User that owns it */
        *group,			/* Group that owns it */
        *attribs,		/* Attribs (-rwxr-x-rx) */
        *destfile;		/* Full pathname to the destination for the 
                                    file transfer */
+
+  int fd;			/* Already open fd for this file */
+  /* FIXME - add fd_open function */
+
   time_t datetime;		/* File date and time */
   off_t size,			/* Size of the file */
         startsize;		/* Size to start the transfer at */
   unsigned int isdir : 1,	/* File type */
                isexe : 1,
                islink : 1,
-               isblock : 1,
-               ischar : 1,
-               issocket : 1,
-               isfifo : 1,
-               ascii : 1, 	/* Transfer in ASCII mode */
                selected : 1,	/* Is this file selected? */
                was_sel : 1,	/* Was this file selected before  */
                shown : 1,	/* Is this file shown? */
@@ -164,21 +210,103 @@ struct gftp_file_tag
                transfer_done : 1, /* Is current file transfer done? */
                is_fd : 1;	/* Is this a file descriptor? */
   char transfer_action;		/* See the GFTP_TRANS_ACTION_* vars above */
-  void *node;			/* Pointer to the node for the gui */
-  int fd;
+  void *user_data;
 };
 
 
 typedef struct gftp_proxy_hosts_tag 
 {
-  gint32 ipv4_network_address,
+  /* FIXME - add IPV4 stuff here */
+
+  gint32 ipv4_network_address, 
          ipv4_netmask;
   char *domain;
 } gftp_proxy_hosts;
 
 
+/* Note, these numbers must match up to the index number in config_file.c
+   in the declaration of gftp_option_types */
+typedef enum 
+{
+  gftp_option_type_text		= 0,
+  gftp_option_type_textcombo	= 1,
+  gftp_option_type_textcomboedt = 2,
+  gftp_option_type_hidetext	= 3,
+  gftp_option_type_int		= 4,
+  gftp_option_type_checkbox	= 5,
+  gftp_option_type_intcombo	= 6,
+  gftp_option_type_float	= 7,
+  gftp_option_type_color	= 8,
+  gftp_option_type_notebook	= 9
+} gftp_option_type_enum;
+
+
+#define GFTP_PORT_GTK			(1 << 1)
+#define GFTP_PORT_TEXT			(1 << 2)
+#define GFTP_PORT_ALL			(GFTP_PORT_GTK | GFTP_PORT_TEXT)
+
+
+typedef struct gftp_config_list_vars_tag
+{
+  char *key;
+  void * (*read_func) (char *buf, int line);
+  void (*write_func) (FILE *fd, void *data);
+  GList * list;
+  unsigned int num_items;
+  char *header;
+} gftp_config_list_vars;
+
+
+#define GFTP_CVARS_FLAGS_DYNMEM			(1 << 1)
+#define GFTP_CVARS_FLAGS_DYNLISTMEM		(1 << 2)
+#define GFTP_CVARS_FLAGS_SHOW_BOOKMARK		(1 << 3)
+
+typedef struct gftp_config_vars_tag
+{
+  char *key,			/* variable name */
+       *description;		/* How this field will show up in the dialog */
+  int otype;			/* Type of option this is */
+  void *value;
+  void *listdata;		/* For options that have several different 
+				   options, this is a list of all the options.
+				   Each option_type that uses this will use this
+				   field differently */
+  int flags;			/* See GFTP_CVARS_FLAGS_* above */
+  char *comment;                /* Comment to write out to the config file */
+  int ports_shown;		/* What ports of gFTP is this option shown in */
+  void *user_data;		/* Data that the GUI can store here (Widget in gtk+) */
+} gftp_config_vars;
+
+
+typedef struct gftp_option_type_tag
+{
+  int (*read_function) (char *str, gftp_config_vars * cv, int line);
+  int (*write_function) (gftp_config_vars * cv, FILE * fd, int to_config_file);
+  void (*copy_function) (gftp_config_vars * cv, gftp_config_vars * dest_cv);
+  int (*compare_function) (gftp_config_vars * cv1, gftp_config_vars * cv2);
+  void *(*ui_print_function) (gftp_config_vars * cv, void *user_data, void *value);
+  void (*ui_save_function) (gftp_config_vars * cv, void *user_data);
+  void (*ui_cancel_function) (gftp_config_vars * cv, void *user_data);
+  void *user_data;
+} gftp_option_type_var;
+
+
+#define GFTP_TEXTCOMBOEDT_EDITABLE 	(1 << 0)
+
+typedef struct gftp_textcomboedt_data_tag
+{
+  char *description,
+       *text;
+  int flags;
+} gftp_textcomboedt_data;
+
+
 typedef struct gftp_request_tag gftp_request;
 
+typedef void (*gftp_logging_func)		( gftp_logging_level level, 
+						  gftp_request * request, 
+						  const char *string, ... );
+
 struct gftp_request_tag 
 {
   int protonum;			/* Current number of the protocol this is 
@@ -188,34 +316,31 @@ struct gftp_request_tag
        *password,		/* Password for host */
        *account,		/* Account for host (FTP only) */
        *directory,		/* Current working directory */
-       *proxy_config,		/* Proxy configuration */
-       *proxy_hostname,		/* Proxy hostname */
-       *proxy_username,		/* Proxy username */
-       *proxy_password,		/* Proxy password */
-       *proxy_account,		/* Proxy account (FTP only) */
        *url_prefix,		/* URL Prefix (ex: ftp) */
-       *protocol_name,		/* Protocol description */
        *last_ftp_response,	/* Last response from server */
        *last_dir_entry;		/* Last dir entry from server */
   size_t last_dir_entry_len;	/* Length of last_dir_entry */
 
-  unsigned int port,		/* Port of remote site */
-               proxy_port;	/* Port of the proxy server */
+  unsigned int port;		/* Port of remote site */
 
-  int sockfd,			/* Control connection (read) */
-      datafd,			/* Data connection */
+  int datafd,			/* Data connection */
       cachefd;			/* For the directory cache */
   int wakeup_main_thread[2];	/* FD that gets written to by the threads
                                    to wakeup the parent */
-        
+
+  /* One of these are used to lookup the IP address of the host we are
+     connecting to */
 #if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
-  struct addrinfo *hostp;	/* Remote host we are connected to */
+  struct addrinfo *hostp;
 #else
-  struct hostent host, *hostp;  /* Remote host we are connected to */
+  struct hostent host, *hostp;
 #endif
 
-  int data_type;		/* ASCII or BINARY (FTP only) */
-  unsigned int use_proxy : 1,           /* Go out of proxy server */
+  int server_type;		/* The type of server we are connected to.
+                                   See GFTP_DIRTYPE_* above */
+  unsigned int free_hostp : 1, 		/* Should we free the hostp structure 
+					   in gftp_destroy_request() */
+               use_proxy : 1,
                always_connected : 1,
                need_hostport : 1,
                need_userpass : 1,
@@ -234,9 +359,18 @@ struct gftp_request_tag
   gftp_logging_func logging_function;
   void *user_data;
 
-  void (*init)				( gftp_request * request );
+  int (*init)				( gftp_request * request );
+  ssize_t (*read_function)		( gftp_request * request,
+					  void *ptr, 
+					  size_t size, 
+					  int fd );
+  ssize_t (*write_function)		( gftp_request * request, 
+					  const char *ptr, 
+					  size_t size, 
+					  int fd );
   void (*destroy)			( gftp_request * request );
   int (*connect)			( gftp_request * request );
+  int (*post_connect)			( gftp_request * request );
   void (*disconnect) 			( gftp_request * request );
   off_t (*get_file) 			( gftp_request * request, 
 					  const char *filename, 
@@ -265,8 +399,6 @@ struct gftp_request_tag
   int (*get_next_file)			( gftp_request * request, 
 					  gftp_file *fle, 
 					  int fd );
-  int (*set_data_type)			( gftp_request * request, 
-					  int data_type );
   off_t (*get_file_size) 		( gftp_request * request, 
 					  const char *filename );
   int (*chdir)				( gftp_request * request, 
@@ -290,29 +422,30 @@ struct gftp_request_tag
 					  const char *filename );
   int (*parse_url)			( gftp_request * request,
 					  const char *url );
-  void (*set_config_options)		( gftp_request * request );
+  int (*set_config_options)		( gftp_request * request );
   void (*swap_socks)			( gftp_request * dest,
 					  gftp_request * source );
 
-  /* Options */
-  gftp_transfer_type transfer_type;	/* Passive or non-passive (FTP only) */
-  int network_timeout,
-      retries,
-      sleep_time,
-      passive_transfer;
-  float maxkbs;
-  char *sftpserv_path;
+  gftp_config_vars * local_options_vars;
+  int num_local_options_vars;
+  GHashTable * local_options_hash;
+#ifdef USE_SSL
+  SSL * ssl;
+#endif
+
+#if GLIB_MAJOR_VERSION > 1
+  GIConv iconv; 
+  unsigned int iconv_initialized : 1;
+#endif
 };
 
 
-typedef struct gftp_transfer_tag gftp_transfer;
-
-struct gftp_transfer_tag
+typedef struct gftp_transfer_tag
 {
   gftp_request * fromreq,
                * toreq;
 
-  unsigned int transfer_direction : 1,
+  unsigned int transfer_direction : 1, /* FIXME - take out */
                cancel : 1,
                ready : 1,
                started : 1,
@@ -347,11 +480,12 @@ struct gftp_transfer_tag
   void * fromwdata,
        * towdata;
 
-  void *statmutex,
-       *structmutex;
-  void *node;
+  GStaticMutex statmutex,
+               structmutex;
+
+  void *user_data;
   void *clist;
-};
+} gftp_transfer;
 
 
 typedef struct gftp_log_tag
@@ -364,14 +498,16 @@ typedef struct gftp_log_tag
 typedef struct supported_gftp_protocols_tag 
 {
   char *name;					/* Description of protocol */
-  void (*init) (gftp_request * request);	/* Init function */
+  int (*init) (gftp_request * request);		/* Init function */
+  void (*register_options) (void);		/* Protocol options */
   char *url_prefix;				/* URL Prefix */
-  int shown;					/* Whether this protocol is shown or not to the user
-                                                   in the protocol dropdown box */
+  int shown;					/* Whether this protocol is 
+                                                   shown or not to the user in 
+                                                   the protocol dropdown box */
 } supported_gftp_protocols;
 
 
-typedef struct gftp_bookmarks_tag gftp_bookmarks;
+typedef struct gftp_bookmarks_tag gftp_bookmarks_var;
 
 struct gftp_bookmarks_tag 
 {
@@ -388,13 +524,14 @@ struct gftp_bookmarks_tag
                isfolder : 1,   /* If this is set, then the children field can
                                   be non-NULL */
                save_password : 1; /* Save this password */
-  gftp_bookmarks *children, 	/* The children of this node. */
-                 *prev, 	/* The parent of this node */
-                 *next; 	/* The next sibling of this node */
+  gftp_bookmarks_var *children, /* The children of this node. */
+                     *prev, 	/* The parent of this node */
+                     *next; 	/* The next sibling of this node */
   gpointer cnode; 
 
-  /* Site options */
-  char *sftpserv_path;		/* Path to the sftp server */
+  gftp_config_vars * local_options_vars;
+  int num_local_options_vars;
+  GHashTable * local_options_hash;
 };
 
 
@@ -416,87 +553,38 @@ typedef struct gftp_color_tag
 } gftp_color;
 
 
-typedef struct gftp_config_vars_tag
-{
-  char *key,			/* variable name */
-       *description;		/* How this field will show up in the dialog */
-  gpointer var;			/* Pointer to our variable */
-  int type;			/* See defines below */
-  char *comment;                /* Comment to write out to the config file */
-  gpointer widget;
-  int ports_shown;		/* What ports of gFTP is this option shown in */
-} gftp_config_vars;
-
-#define CONFIG_INTTEXT                  1
-#define CONFIG_FLOATTEXT                2
-#define CONFIG_CHECKBOX                 3       
-#define CONFIG_LABEL                    4
-#define CONFIG_NOTEBOOK                 5
-#define CONFIG_HIDEINT                  6
-#define CONFIG_TABLE			7
-#define CONFIG_CHARTEXT                 8
-#define CONFIG_COMBO			9
-#define CONFIG_TEXT			10
-#define CONFIG_COLOR			11
-#define CONFIG_UINTTEXT                 12
-#define CONFIG_CHARPASS			13
-
-#define GFTP_PORT_GTK			(1 << 1)
-#define GFTP_PORT_TEXT			(1 << 2)
-#define GFTP_PORT_ALL			(GFTP_PORT_GTK | GFTP_PORT_TEXT)
-
-typedef struct gftp_proxy_type_tag
-{
-  char *key,
-       *description;
-} gftp_proxy_type;
-
-#define GFTP_CUSTOM_PROXY_NUM        8
-
 typedef struct gftp_getline_buffer_tag
 {
   char *buffer,
        *curpos;
   size_t max_bufsize,
          cur_bufsize;
+  unsigned int eof : 1;
 } gftp_getline_buffer;
 
-/* Global config options */
+
+/* Global config options. These are defined in options.h */
+extern GList * gftp_file_transfers, * gftp_file_transfer_logs,
+             * gftp_options_list;
+extern GHashTable * gftp_global_options_htable, * gftp_bookmarks_htable, 
+                  * gftp_config_list_htable;
+extern gftp_config_vars gftp_global_config_vars[];
 extern supported_gftp_protocols gftp_protocols[];
-extern char version[], *emailaddr, *edit_program, *view_program, 
-            *firewall_host, *firewall_username, *firewall_password, 
-            *firewall_account, *proxy_config, *http_proxy_host, 
-            *http_proxy_username, *http_proxy_password, 
-            *startup_directory, *ssh_prog_name, *ssh_extra_params, 
-            **ssh_extra_params_list, *default_protocol, *ssh1_sftp_path, 
-            *ssh2_sftp_path;
-extern int num_ssh_extra_params;
-extern FILE * logfd;
-extern double maxkbs;
-extern GList * proxy_hosts, * registered_exts, * viewedit_processes, 
-             * file_transfers, * file_transfer_logs;
-extern gftp_bookmarks * bookmarks;
-extern int do_one_transfer_at_a_time, start_file_transfers, 
-           transfer_in_progress, passive_transfer, sort_dirs_first, 
-           show_hidden_files, refresh_files, listbox_local_width, 
-           listbox_remote_width, listbox_file_height, transfer_height, 
-           log_height, retries, sleep_time, network_timeout, use_http11, 
-           listbox_dblclick_action, file_trans_column, local_columns[6], 
-           remote_columns[6], resolve_symlinks, firewall_port, http_proxy_port,
-           overwrite_by_default, append_file_transfers, enable_old_ssh, 
-           ssh_need_userpass, ssh_use_askpass, sshv2_use_sftp_subsys, 
-           local_sortcol, local_sortasds, remote_sortcol, remote_sortasds;
-extern guint max_log_window_size;
-extern GHashTable * bookmarks_htable, * config_htable;
-extern GList * localhistory, * remotehistory, * host_history, * port_history, 
-             * user_history;
-extern unsigned int host_len, port_len, user_len, localhistlen, remotehistlen;
-extern volatile sig_atomic_t viewedit_process_done;
-extern gftp_config_vars config_file_vars[];
-extern gftp_proxy_type proxy_type[];
-extern gftp_color send_color, recv_color, error_color, misc_color;
+extern gftp_bookmarks_var * gftp_bookmarks;
+extern char gftp_version[];
+extern FILE * gftp_logfd;
+extern int gftp_configuration_changed;
+
+/* This is defined in config_file.c */
+
+extern gftp_option_type_var gftp_option_types[];
 
 /* cache.c */
+void gftp_generate_cache_description 	( gftp_request * request, 
+					  char *description,
+					  size_t len, 
+					  int ignore_directory );
+
 int gftp_new_cache_entry 		( gftp_request * request );
 
 int gftp_find_cache_entry 		( gftp_request * request );
@@ -504,32 +592,72 @@ int gftp_find_cache_entry 		( gftp_request * request );
 void gftp_clear_cache_files 		( void );
 
 void gftp_delete_cache_entry 		( gftp_request * request,
+					  char *descr,
 					  int ignore_directory );
 
 /* config_file.c */
-void gftp_read_config_file 		( char **argv,
-					  int get_xpms );
+int gftp_config_parse_args 		( char *str, 
+					  int numargs, 
+					  int lineno, 
+					  char **first, 
+					  ... );
 
-void gftp_read_bookmarks 		( void );
+void gftp_add_bookmark 			( gftp_bookmarks_var * newentry );
 
-void add_to_bookmark                    ( gftp_bookmarks *newentry );
-
-void gftp_write_config_file 		( void );
+void gftp_read_config_file 		( char *global_data_path );
 
 void gftp_write_bookmarks_file 		( void );
 
-GHashTable * build_bookmarks_hash_table	( gftp_bookmarks * entry );
+void gftp_write_config_file 		( void );
 
-void print_bookmarks 			( gftp_bookmarks * bookmarks );
+GHashTable * build_bookmarks_hash_table	( gftp_bookmarks_var * entry );
+
+void print_bookmarks 			( gftp_bookmarks_var * bookmarks );
+
+void gftp_lookup_global_option 		( char * key, 
+					  void *value );
+
+void gftp_lookup_request_option 	( gftp_request * request, 
+					  char * key, 
+					  void *value );
+
+void gftp_lookup_bookmark_option 	( gftp_bookmarks_var * bm, 
+					  char * key, 
+					  void *value );
+
+void gftp_set_global_option 		( char * key, 
+					  const void *value );
+
+void gftp_set_request_option 		( gftp_request * request, 
+					  char * key, 
+					  const void *value );
+
+void gftp_set_bookmark_option 		( gftp_bookmarks_var * bm,
+					  char * key, 
+					  const void *value );
+
+void gftp_register_config_vars 		( gftp_config_vars *config_vars );
+
+void gftp_copy_local_options 		( gftp_config_vars ** new_options_vars, 
+					  GHashTable ** new_options_hash,
+					  gftp_config_vars * orig_options,
+					  int num_local_options_vars );
+
+void gftp_config_free_options 		( gftp_config_vars * options_vars,
+					  GHashTable * options_hash,
+					  int num_options_vars );
+
+void gftp_bookmarks_destroy 		( gftp_bookmarks_var * bookmarks );
+
+void gftp_free_proxy_hosts 		( GList * proxy_hosts );
+
+GList * gftp_copy_proxy_hosts 		( GList * proxy_hosts );
 
 /* misc.c */
 char *insert_commas 			( off_t number, 
 					  char *dest_str, 
 					  size_t dest_len );
 
-long file_countlf 			( int filefd, 
-					  long endpos );
-
 char *alltrim 				( char *str );
 
 char *expand_path 			( const char *src );
@@ -541,9 +669,6 @@ void make_nonnull 			( char **str );
 int copyfile				( char *source,
 					  char *dest );
 
-char *get_xpm_path 			( char *filename, 
-					  int quit_on_err );
-
 int gftp_match_filespec 		( char *filename, 
 					  char *filespec );
 
@@ -567,88 +692,81 @@ int compare_request 			( gftp_request * request1,
 					  gftp_request * request2, 
 					  int compare_dirs );
 
+gftp_transfer * gftp_tdata_new 		( void );
+
 void free_tdata 			( gftp_transfer * tdata );
 
-gftp_request * copy_request 		( gftp_request * req );
-
-int ptym_open 				( char *pts_name );
-
-int ptys_open 				( int fdm, 
-					  char *pts_name );
-
-int tty_raw 				( int fd );
-
-char **make_ssh_exec_args 		( gftp_request * request, 
-					  char *execname,
-					  int use_sftp_subsys,
-					  char *portstring );
-
-char * ssh_start_login_sequence 	( gftp_request * request, 
-					  int fd );
-
-#ifdef G_HAVE_GINT64
-gint64 hton64				( gint64 val );
-#endif
+gftp_request * copy_request 		( gftp_request * req,
+					  int copy_local_options );
 
 GList * gftp_sort_filelist 		( GList * filelist, 
 					  int column, 
 					  int asds );
 
+mode_t gftp_parse_attribs 		( char *attribs );
+
+char * gftp_gen_ls_string 		( gftp_file * fle, 
+					  char *file_prefixstr, 
+					  char *file_suffixstr );
+
+char * base64_encode 			( char *str );
+
+void gftp_free_bookmark 		( gftp_bookmarks_var * entry );
+
+void gftp_shutdown			( void );
+
+GList * get_next_selection 		( GList * selection, 
+					  GList ** list, 
+					  int *curnum );
+
+char * gftp_build_path 			( const char *first_element,
+					  ... );
+
+off_t gftp_parse_file_size 		( char *str );
+
 /* protocols.c */
-#define GFTP_GET_HOSTNAME(request)		(request->hostname)
-#define GFTP_GET_USERNAME(request)		(request->username)
-#define GFTP_GET_PASSWORD(request)		(request->password)
-#define GFTP_GET_ACCOUNT(request)		(request->account)
-#define GFTP_GET_DIRECTORY(request)		(request->directory)
-#define GFTP_GET_PORT(request)			(request->port)
-#define GFTP_GET_PROXY_CONFIG(request)		(request->proxy_config)
-#define GFTP_GET_PROXY_HOSTNAME(request)	(request->proxy_hostname)
-#define GFTP_GET_PROXY_USERNAME(request)	(request->proxy_username)
-#define GFTP_GET_PROXY_PASSWORD(request)	(request->proxy_password)
-#define GFTP_GET_PROXY_ACCOUNT(request)		(request->proxy_account)
-#define GFTP_GET_PROXY_PORT(request)		(request->proxy_port)
-#define GFTP_GET_URL_PREFIX(request)		(request->url_prefix)
-#define GFTP_GET_PROTOCOL_NAME(request)		(request->protocol_name)
-#define GFTP_GET_LAST_RESPONSE(request)		(request->last_ftp_response)
-#define GFTP_GET_LAST_DIRENT(request)		(request->last_dir_entry)
-#define GFTP_GET_LAST_DIRENT_LEN(request)	(request->last_dir_entry_len)
-#define GFTP_GET_CONTROL_FD(request)		(request->sockfd)
-#define GFTP_GET_DATA_FD(request)		(request->datafd)
-#define GFTP_GET_DATA_TYPE(request)		(request->data_type)
-#define GFTP_GET_TRANSFER_TYPE(request)		(request->transfer_type)
-#define GFTP_SET_TRANSFER_TYPE(request,val) 	(request->transfer_type = (val))
-#define GFTP_GET_LOGGING(request)		(request->logging)
-#define GFTP_SET_LOGGING(request, val)		(request->logging = (val))
-#define GFTP_UNSAFE_SYMLINKS(request)		(request->unsafe_symlinks)
 #define GFTP_FTP_NUM				0
 #define GFTP_HTTP_NUM				1
-#define GFTP_LOCAL_NUM				2
-#define GFTP_SSHV2_NUM				3
-#define GFTP_BOOKMARK_NUM			4
-#define GFTP_SSH_NUM				5
-#define GFTP_TYPE_BINARY			1
-#define GFTP_TYPE_ASCII				2   
+#define GFTP_HTTPS_NUM				2
+#define GFTP_LOCAL_NUM				3
+#define GFTP_SSHV2_NUM				4
+#define GFTP_BOOKMARK_NUM			5
+
 #define GFTP_IS_CONNECTED(request)		((request) != NULL && \
-                                                 ((request)->sockfd > 0 || \
+                                                 ((request)->datafd > 0 || \
                                                   (request)->cached || \
                                                   (request)->always_connected))
 
 
-void rfc959_init 			( gftp_request * request );
+int rfc959_init 			( gftp_request * request );
+
+void rfc959_register_module		( void );
 
 int rfc959_get_next_file 		( gftp_request * request, 
 					  gftp_file *fle, 
 					  int fd );
 
-void rfc2068_init 			( gftp_request * request );
+int rfc2068_init 			( gftp_request * request );
 
-void local_init 			( gftp_request * request );
+void rfc2068_register_module		( void );
 
-void ssh_init 				( gftp_request * request );
+int https_init 				( gftp_request * request );
 
-void sshv2_init 			( gftp_request * request );
+void https_register_module		( void );
 
-void bookmark_init 			( gftp_request * request );
+int local_init 				( gftp_request * request );
+
+void local_register_module		( void );
+
+int sshv2_init 				( gftp_request * request );
+
+void sshv2_register_module		( void );
+
+void ssl_register_module		( void );
+
+int bookmark_init 			( gftp_request * request );
+
+void bookmark_register_module		( void );
 
 gftp_request *gftp_request_new 		( void );
 
@@ -664,22 +782,22 @@ void gftp_disconnect 			( gftp_request * request );
 off_t gftp_get_file 			( gftp_request * request, 
 					  const char *filename, 
 					  int fd,
-					  size_t startsize );
+					  off_t startsize );
 
 int gftp_put_file 			( gftp_request * request, 
 					  const char *filename, 
 					  int fd,
-					  size_t startsize,
-					  size_t totalsize );
+					  off_t startsize,
+					  off_t totalsize );
 
-long gftp_transfer_file 		( gftp_request *fromreq, 
+off_t gftp_transfer_file 		( gftp_request *fromreq, 
 					  const char *fromfile, 
 					  int fromfd,
-					  size_t fromsize, 
+					  off_t fromsize, 
 					  gftp_request *toreq, 
 					  const char *tofile, 
 					  int tofd,
-					  size_t tosize );
+					  off_t tosize );
 
 ssize_t gftp_get_next_file_chunk 	( gftp_request * request, 
 					  char *buf, 
@@ -691,6 +809,15 @@ ssize_t gftp_put_next_file_chunk 	( gftp_request * request,
 
 int gftp_list_files 			( gftp_request * request );
 
+#if GLIB_MAJOR_VERSION > 1
+char * gftp_string_to_utf8		( gftp_request * request, 
+					  char *str );
+#endif
+
+int gftp_parse_bookmark 		( gftp_request * request, 
+					  gftp_request * local_request,
+					  const char * bookmark );
+
 int gftp_parse_url 			( gftp_request * request, 
 					  const char *url );
 
@@ -702,11 +829,6 @@ int gftp_end_transfer 			( gftp_request * request );
 
 int gftp_abort_transfer 		( gftp_request * request );
 
-int gftp_read_response 			( gftp_request * request );
-
-int gftp_set_data_type 			( gftp_request * request, 
-					  int data_type );
-
 void gftp_set_hostname 			( gftp_request * request, 
 					  const char *hostname );
 
@@ -725,21 +847,6 @@ int gftp_set_directory 			( gftp_request * request,
 void gftp_set_port 			( gftp_request * request, 
 					  unsigned int port );
 
-void gftp_set_proxy_hostname 		( gftp_request * request, 
-					  const char *hostname );
-
-void gftp_set_proxy_username 		( gftp_request * request, 
-					  const char *username );
-
-void gftp_set_proxy_password 		( gftp_request * request, 
-					  const char *password );
-
-void gftp_set_proxy_account 		( gftp_request * request, 
-					  const char *account );
-
-void gftp_set_proxy_port 		( gftp_request * request, 
-					  unsigned int port );
-
 int gftp_remove_directory 		( gftp_request * request, 
 					  const char *directory );
 
@@ -764,43 +871,29 @@ int gftp_set_file_time 			( gftp_request * request,
 char gftp_site_cmd 			( gftp_request * request, 
 					  const char *command );
 
-void gftp_set_proxy_config 		( gftp_request * request, 
-					  const char *proxy_config );
-
 off_t gftp_get_file_size 		( gftp_request * request, 
 					  const char *filename );
 
-int gftp_need_proxy 			( gftp_request * request,
-					  char *service );
-
-char *gftp_convert_ascii 		( char *buf, 
-					  ssize_t *len, 
-					  int direction );
-
 void gftp_calc_kbs 			( gftp_transfer * tdata, 
 				 	  ssize_t num_read );
 
-int gftp_parse_ls 			( const char *lsoutput, 
+time_t parse_time 			( char *str,
+					  char **endpos );
+
+int gftp_parse_ls 			( gftp_request * request,
+					  const char *lsoutput, 
 					  gftp_file *fle );
 
 int gftp_get_all_subdirs 		( gftp_transfer * transfer, 
 					  void (*update_func) 
 						( gftp_transfer * transfer ));
 
-int gftp_get_file_transfer_mode 	( char *filename,
-					  int def );
-
 int gftp_connect_server 		( gftp_request * request, 
-					  char *service );
+					  char *service,
+					  char *proxy_hostname,
+					  int proxy_port );
 
-void gftp_set_sftpserv_path		( gftp_request * request,
-					  char *path );
-
-#if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
-
-int get_port 				( struct addrinfo *addr );
-
-#else
+#if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
 
 struct hostent *r_gethostbyname 	( const char *name, 
 					  struct hostent *result_buf, 
@@ -812,22 +905,25 @@ struct servent *r_getservbyname 	( const char *name,
 					  const char *proto,
 					  struct servent *result_buf, 
 					  int *h_errnop );
-void gftp_set_config_options 		( gftp_request * request );
+
+int gftp_set_config_options 		( gftp_request * request );
 
 void print_file_list 			( GList * list );
 
+void gftp_free_getline_buffer 		( gftp_getline_buffer ** rbuf );
+
 ssize_t gftp_get_line 			( gftp_request * request, 
 					  gftp_getline_buffer ** rbuf,
 					  char * str, 
 					  size_t len, 
 					  int fd );
 
-ssize_t gftp_read 			( gftp_request * request, 
+ssize_t gftp_fd_read 			( gftp_request * request, 
 					  void *ptr, 
 					  size_t size, 
 					  int fd );
 
-ssize_t gftp_write 			( gftp_request * request, 
+ssize_t gftp_fd_write 			( gftp_request * request, 
 					  const char *ptr, 
 					  size_t size, 
 					  int fd );
@@ -837,12 +933,53 @@ ssize_t gftp_writefmt 			( gftp_request * request,
 					  const char *fmt, 
 					  ... );
 
-int gftp_set_sockblocking 		( gftp_request * request, 
+int gftp_fd_set_sockblocking 		( gftp_request * request, 
 					  int fd, 
 					  int non_blocking );
 
 void gftp_swap_socks 			( gftp_request * dest, 
 					  gftp_request * source );
 
+void gftp_calc_kbs 			( gftp_transfer * tdata, 
+					  ssize_t num_read );
+
+int gftp_get_transfer_status 		( gftp_transfer * tdata, 
+					  ssize_t num_read );
+
+int gftp_fd_open 			( gftp_request * request, 
+					  const char *pathname, 
+					  int flags,
+					  mode_t perms );
+
+/* pty.c */
+char * gftp_get_pty_impl 		( void );
+
+pid_t gftp_exec_with_new_pty 		( gftp_request * request, 
+					  int *fdm, 
+					  char **args );
+
+pid_t gftp_exec_without_new_pty 	( gftp_request * request, 
+					  int *fdm, 
+					  char **args );
+
+#ifdef USE_SSL
+/* sslcommon.c */
+int gftp_ssl_startup 			( gftp_request * request );
+
+int gftp_ssl_session_setup 		( gftp_request * request );
+
+void gftp_ssl_free 			( gftp_request * request );
+
+ssize_t gftp_ssl_read 			( gftp_request * request, 
+					  void *ptr, 
+					  size_t size, 
+					  int fd );
+
+ssize_t gftp_ssl_write 			( gftp_request * request, 
+					  const char *ptr, 
+					  size_t size, 
+					  int fd );
+#endif /* USE_SSL */
+
 #endif
 
