@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  local.c - functions that will use the local system                       */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp.h"
-static const char cvsid[] = "$Id: local.c,v 1.13 2002/11/27 02:23:51 masneyb Exp $";
+static const char cvsid[] = "$Id: local.c,v 1.26 2003/08/12 01:05:02 masneyb Exp $";
 
 typedef struct local_protocol_data_tag
 {
@@ -56,14 +56,14 @@ local_connect (gftp_request * request)
 {
   char tempstr[PATH_MAX];
 
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
 
   if (request->directory)
     {
       if (chdir (request->directory) != 0)
         {
-          request->logging_function (gftp_logging_error, request->user_data,
+          request->logging_function (gftp_logging_error, request,
                              _("Could not change local directory to %s: %s\n"),
                              request->directory, g_strerror (errno));
         }
@@ -74,11 +74,10 @@ local_connect (gftp_request * request)
   if (getcwd (tempstr, sizeof (tempstr)) != NULL)
     {
       tempstr[sizeof (tempstr) - 1] = '\0';
-      request->directory = g_malloc (strlen (tempstr) + 1);
-      strcpy (request->directory, tempstr);
+      request->directory = g_strdup (tempstr);
     }
   else
-    request->logging_function (gftp_logging_error, request->user_data,
+    request->logging_function (gftp_logging_error, request,
                              _("Could not get current working directory: %s\n"),
                              g_strerror (errno));
 
@@ -94,8 +93,8 @@ local_disconnect (gftp_request * request)
 
   if (request->datafd != -1)
     {
-      if (close (request->datafd) < 0)
-        request->logging_function (gftp_logging_error, request->user_data,
+      if (close (request->datafd) == -1)
+        request->logging_function (gftp_logging_error, request,
                                    _("Error closing file descriptor: %s\n"),
                                    g_strerror (errno));
       request->datafd = -1;
@@ -110,9 +109,9 @@ local_get_file (gftp_request * request, const char *filename, int fd,
   off_t size;
   int flags;
 
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
-  g_return_val_if_fail (filename != NULL, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
 
   if (fd <= 0)
     {
@@ -121,33 +120,28 @@ local_get_file (gftp_request * request, const char *filename, int fd,
       flags |= O_LARGEFILE;
 #endif
 
-      if ((request->datafd = open (filename, flags)) < 0)
-        {
-          request->logging_function (gftp_logging_error, request->user_data,
-                                   _("Error: Cannot open local file %s: %s\n"),
-                                   filename, g_strerror (errno));
-          return (-2);
-        }
+      if ((request->datafd = gftp_fd_open (request, filename, flags, 0)) == -1)
+        return (GFTP_ERETRYABLE); 
     }
   else
     request->datafd = fd;
 
   if ((size = lseek (request->datafd, 0, SEEK_END)) == -1)
     {
-      request->logging_function (gftp_logging_error, request->user_data,
+      request->logging_function (gftp_logging_error, request,
                                  _("Error: Cannot seek on file %s: %s\n"),
                                  filename, g_strerror (errno));
       gftp_disconnect (request);
-      return (-1);
+      return (GFTP_ERETRYABLE);
     }
 
   if (lseek (request->datafd, startsize, SEEK_SET) == -1)
     {
-      request->logging_function (gftp_logging_error, request->user_data,
+      request->logging_function (gftp_logging_error, request,
                                  _("Error: Cannot seek on file %s: %s\n"),
                                  filename, g_strerror (errno));
       gftp_disconnect (request);
-      return (-1);
+      return (GFTP_ERETRYABLE);
     }
 
   return (size);
@@ -160,9 +154,9 @@ local_put_file (gftp_request * request, const char *filename, int fd,
 {
   int flags;
 
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
-  g_return_val_if_fail (filename != NULL, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
 
   if (fd <= 0)
     {
@@ -173,33 +167,28 @@ local_put_file (gftp_request * request, const char *filename, int fd,
       flags |= O_LARGEFILE;
 #endif
 
-      if ((request->datafd = open (filename, flags, S_IRUSR | S_IWUSR)) < 0)
-        {
-          request->logging_function (gftp_logging_error, request->user_data,
-                                   _("Error: Cannot open local file %s: %s\n"),
-                                   filename, g_strerror (errno));
-          return (-2);
-        }
+      if ((request->datafd = gftp_fd_open (request, filename, flags, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
+        return (GFTP_ERETRYABLE);
     }
   else
     request->datafd = fd;
 
   if (ftruncate (request->datafd, startsize) == -1)
     {
-      request->logging_function (gftp_logging_error, request->user_data,
+      request->logging_function (gftp_logging_error, request,
                                _("Error: Cannot truncate local file %s: %s\n"),
                                filename, g_strerror (errno));
       gftp_disconnect (request);
-      return (-1);
+      return (GFTP_ERETRYABLE);
     }
     
   if (lseek (request->datafd, startsize, SEEK_SET) == -1)
     {
-      request->logging_function (gftp_logging_error, request->user_data,
+      request->logging_function (gftp_logging_error, request,
                                  _("Error: Cannot seek on file %s: %s\n"),
                                  filename, g_strerror (errno));
       gftp_disconnect (request);
-      return (-2);
+      return (GFTP_ERETRYABLE);
     }
   return (0);
 }
@@ -219,8 +208,8 @@ local_end_transfer (gftp_request * request)
 
   if (request->datafd > 0)
     {
-      if (close (request->datafd) < 0)
-        request->logging_function (gftp_logging_error, request->user_data,
+      if (close (request->datafd) == -1)
+        request->logging_function (gftp_logging_error, request,
                                    _("Error closing file descriptor: %s\n"),
                                    g_strerror (errno));
 
@@ -232,7 +221,7 @@ local_end_transfer (gftp_request * request)
 
 
 static char *
-make_text_mode (gftp_file * fle, mode_t mode)
+make_text_mode (mode_t mode)
 {
   char *str;
 
@@ -243,40 +232,22 @@ make_text_mode (gftp_file * fle, mode_t mode)
     str[0] = '-';
 
   if (S_ISLNK (mode))
-    {
-      fle->islink = 1; 
-      str[0] = 'l';
-    }
+    str[0] = 'l';
 
   if (S_ISBLK (mode))
-     {
-       fle->isblock = 1;
-       str[0] = 'b';
-     }
+    str[0] = 'b';
 
   if (S_ISCHR (mode))
-     {
-       fle->ischar = 1;
-       str[0] = 'c';
-    }
+    str[0] = 'c';
 
   if (S_ISFIFO (mode))
-    {
-      fle->isfifo = 1;
-      str[0] = 'p';
-    }
+    str[0] = 'p';
 
   if (S_ISSOCK (mode))
-    {
-      fle->issocket = 1;
-      str[0] = 's';
-    }
+    str[0] = 's';
 
   if (S_ISDIR (mode))
-    {
-      fle->isdir = 1;
-      str[0] = 'd';
-    }
+    str[0] = 'd';
 
   str[1] = mode & S_IRUSR ? 'r' : '-';
   str[2] = mode & S_IWUSR ? 'w' : '-';
@@ -321,81 +292,76 @@ static int
 local_get_next_file (gftp_request * request, gftp_file * fle, int fd)
 {
   local_protocol_data * lpd;
+  struct stat st, fst;
   struct dirent *dirp;
   char *user, *group;
   struct passwd *pw;
   struct group *gr;
-  struct stat st;
 
   /* the struct passwd and struct group are not thread safe. But,
      we're ok here because I have threading turned off for the local
      protocol (see use_threads in local_init above) */
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
-  g_return_val_if_fail (fle != NULL, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (fle != NULL, GFTP_EFATAL);
 
   lpd = request->protocol_data;
+
+  g_return_val_if_fail (lpd != NULL, GFTP_EFATAL);
+
   memset (fle, 0, sizeof (*fle));
 
   if ((dirp = readdir (lpd->dir)) == NULL)
     {
       closedir (lpd->dir);
       lpd->dir = NULL;
-      return (-2);
+      return (GFTP_ERETRYABLE);
     }
 
-  fle->file = g_malloc (strlen (dirp->d_name) + 1);
-  strcpy (fle->file, dirp->d_name);
+  fle->file = g_strdup (dirp->d_name);
   if (lstat (fle->file, &st) != 0)
     {
       closedir (lpd->dir);
       lpd->dir = NULL;
-      return (-2);
+      return (GFTP_ERETRYABLE);
+    }
+
+  if (stat (fle->file, &fst) != 0)
+    {
+      closedir (lpd->dir);
+      lpd->dir = NULL;
+      return (GFTP_ERETRYABLE);
     }
 
   if ((user = g_hash_table_lookup (lpd->userhash, 
                                    GUINT_TO_POINTER(st.st_uid))) != NULL)
-    {
-      fle->user = g_malloc (strlen (user) + 1);
-      strcpy (fle->user, user);
-    }
+    fle->user = g_strdup (user);
   else
     {
       if ((pw = getpwuid (st.st_uid)) == NULL)
         fle->user = g_strdup_printf ("%u", st.st_uid); 
       else
-        {
-          fle->user = g_malloc (strlen (pw->pw_name) + 1);
-          strcpy (fle->user, pw->pw_name);
-        }
+        fle->user = g_strdup (pw->pw_name);
 
-      user = g_malloc (strlen (fle->user) + 1);
-      strcpy (user, fle->user);
+      user = g_strdup (fle->user);
       g_hash_table_insert (lpd->userhash, GUINT_TO_POINTER (st.st_uid), user);
     }
 
   if ((group = g_hash_table_lookup (lpd->grouphash, 
                                     GUINT_TO_POINTER(st.st_gid))) != NULL)
-    {
-      fle->group = g_malloc (strlen (group) + 1);
-      strcpy (fle->group, group);
-    }
+    fle->group = g_strdup (group);
   else
     {
       if ((gr = getgrgid (st.st_gid)) == NULL)
         fle->group = g_strdup_printf ("%u", st.st_gid); 
       else
-        {
-          fle->group = g_malloc (strlen (gr->gr_name) + 1);
-          strcpy (fle->group, gr->gr_name);
-        }
+        fle->group = g_strdup (gr->gr_name);
 
-      group = g_malloc (strlen (fle->group) + 1);
-      strcpy (group, fle->group);
+      group = g_strdup (fle->group);
       g_hash_table_insert (lpd->grouphash, GUINT_TO_POINTER (st.st_gid), group);
     }
 
-  fle->attribs = make_text_mode (fle, st.st_mode);
+  fle->attribs = make_text_mode (st.st_mode);
   fle->datetime = st.st_mtime;
 
   if ((fle->attribs[0] == 'b' || fle->attribs[0] == 'u' ||
@@ -404,20 +370,12 @@ local_get_next_file (gftp_request * request, gftp_file * fle, int fd)
   else
     fle->size = st.st_size;
 
-  if (*fle->attribs == 'd')
-    fle->isdir = 1;
-  if (*fle->attribs == 'l')
-    fle->islink = 1;
-  if (strchr (fle->attribs, 'x') != NULL && !fle->isdir && !fle->islink)
-    fle->isexe = 1;
-  if (*fle->attribs == 'b')
-    fle->isblock = 1;
-  if (*fle->attribs == 'c')
-    fle->ischar = 1;
-  if (*fle->attribs == 's')
-    fle->issocket = 1;
-  if (*fle->attribs == 'p')
-    fle->isfifo = 1;
+  fle->isdir = S_ISDIR (fst.st_mode);
+  fle->islink = S_ISLNK (st.st_mode);
+  fle->isexe = (fst.st_mode & S_IXUSR) || 
+               (fst.st_mode & S_IXGRP) ||
+               (fst.st_mode & S_IXOTH);
+
   return (1);
 }
 
@@ -427,23 +385,40 @@ local_list_files (gftp_request * request)
 {
   local_protocol_data *lpd;
   char *tempstr;
+  int freeit;
+
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->directory != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
 
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
   lpd = request->protocol_data;
 
-  tempstr = g_strconcat (request->directory, "/", NULL);
+  g_return_val_if_fail (lpd != NULL, GFTP_EFATAL);
+
+  if (request->directory[strlen (request->directory) - 1] != '/')
+    {
+      tempstr = g_strconcat (request->directory, "/", NULL);
+      freeit = 1;
+    }
+  else
+    {
+      tempstr = request->directory;
+      freeit = 0;
+    }
 
   if ((lpd->dir = opendir (tempstr)) == NULL)
     {
-      request->logging_function (gftp_logging_error, request->user_data,
+      request->logging_function (gftp_logging_error, request,
                            _("Could not get local directory listing %s: %s\n"),
                            tempstr, g_strerror (errno));
-      g_free (tempstr);
-      return (-1);
+      if (freeit)
+        g_free (tempstr);
+      return (GFTP_ERETRYABLE);
     }
 
-  g_free (tempstr);
+  if (freeit)
+    g_free (tempstr);
+
   return (0);
 }
 
@@ -454,7 +429,7 @@ local_get_file_size (gftp_request * request, const char *filename)
   struct stat st;
 
   if (stat (filename, &st) == -1)
-    return (-1);
+    return (GFTP_ERETRYABLE);
   return (st.st_size);
 }
 
@@ -464,57 +439,62 @@ local_chdir (gftp_request * request, const char *directory)
 {
   char tempstr[255];
 
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
-  g_return_val_if_fail (directory != NULL, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
 
   if (chdir (directory) == 0)
     {
-      request->logging_function (gftp_logging_misc, request->user_data,
+      request->logging_function (gftp_logging_misc, request,
                           _("Successfully changed local directory to %s\n"),
                           directory);
+
       if (request->directory != directory)
         {
           if (getcwd (tempstr, sizeof (tempstr)) == NULL)
-	    {
-              request->logging_function (gftp_logging_error, request->user_data,
-                            _("Could not get current working directory: %s\n"),
-                            g_strerror (errno));
-	      return (-1);
-	    }
+            {
+              request->logging_function (gftp_logging_error, request,
+                             _("Could not get current working directory: %s\n"),
+                             g_strerror (errno));
+	      return (GFTP_ERETRYABLE);
+            }
+
           if (request->directory)
-	    g_free (request->directory);
-          request->directory = g_malloc (strlen (tempstr) + 1);
-          strcpy (request->directory, tempstr);
+            g_free (request->directory);
+          request->directory = g_strdup (tempstr);
         }
+
       return (0);
     }
-  request->logging_function (gftp_logging_error, request->user_data,
-                             _("Could not change local directory to %s: %s\n"),
-                             directory, g_strerror (errno));
-  return (-1);
+  else
+    {
+      request->logging_function (gftp_logging_error, request,
+                              _("Could not change local directory to %s: %s\n"),
+                              directory, g_strerror (errno));
+      return (GFTP_ERETRYABLE);
+    }
 }
 
 
 static int
 local_rmdir (gftp_request * request, const char *directory)
 {
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
-  g_return_val_if_fail (directory != NULL, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
 
   if (rmdir (directory) == 0)
     {
-      request->logging_function (gftp_logging_misc, request->user_data,
+      request->logging_function (gftp_logging_misc, request,
                                  _("Successfully removed %s\n"), directory);
       return (0);
     }
   else
     {
-      request->logging_function (gftp_logging_error, request->user_data,
+      request->logging_function (gftp_logging_error, request,
                               _("Error: Could not remove directory %s: %s\n"),
                               directory, g_strerror (errno));
-      return (-1);
+      return (GFTP_ERETRYABLE);
     }
 }
 
@@ -522,22 +502,22 @@ local_rmdir (gftp_request * request, const char *directory)
 static int
 local_rmfile (gftp_request * request, const char *file)
 {
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
-  g_return_val_if_fail (file != NULL, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (file != NULL, GFTP_EFATAL);
 
   if (unlink (file) == 0)
     {
-      request->logging_function (gftp_logging_misc, request->user_data,
+      request->logging_function (gftp_logging_misc, request,
                                  _("Successfully removed %s\n"), file);
       return (0);
     }
   else
     {
-      request->logging_function (gftp_logging_error, request->user_data,
+      request->logging_function (gftp_logging_error, request,
                                  _("Error: Could not remove file %s: %s\n"),
                                  file, g_strerror (errno));
-      return (-1);
+      return (GFTP_ERETRYABLE);
     }
 }
 
@@ -545,23 +525,23 @@ local_rmfile (gftp_request * request, const char *file)
 static int
 local_mkdir (gftp_request * request, const char *directory)
 {
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
-  g_return_val_if_fail (directory != NULL, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
 
   if (mkdir (directory, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) == 0)
     {
-      request->logging_function (gftp_logging_misc, request->user_data,
+      request->logging_function (gftp_logging_misc, request,
                                  _("Successfully made directory %s\n"),
                                  directory);
       return (0);
     }
   else
     {
-      request->logging_function (gftp_logging_error, request->user_data,
+      request->logging_function (gftp_logging_error, request,
                                  _("Error: Could not make directory %s: %s\n"),
                                  directory, g_strerror (errno));
-      return (-1);
+      return (GFTP_ERETRYABLE);
     }
 }
 
@@ -570,24 +550,24 @@ static int
 local_rename (gftp_request * request, const char *oldname,
 	      const char *newname)
 {
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
-  g_return_val_if_fail (oldname != NULL, -2);
-  g_return_val_if_fail (newname != NULL, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (oldname != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (newname != NULL, GFTP_EFATAL);
 
   if (rename (oldname, newname) == 0)
     {
-      request->logging_function (gftp_logging_misc, request->user_data,
+      request->logging_function (gftp_logging_misc, request,
                                  _("Successfully renamed %s to %s\n"),
                                  oldname, newname);
       return (0);
     }
   else
     {
-      request->logging_function (gftp_logging_error, request->user_data,
+      request->logging_function (gftp_logging_error, request,
                                  _("Error: Could not rename %s to %s: %s\n"),
                                  oldname, newname, g_strerror (errno));
-      return (-1);
+      return (GFTP_ERETRYABLE);
     }
 }
 
@@ -598,26 +578,26 @@ local_chmod (gftp_request * request, const char *file, int mode)
   char buf[10];
   int newmode;
 
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
-  g_return_val_if_fail (file != NULL, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (file != NULL, GFTP_EFATAL);
 
   g_snprintf (buf, sizeof (buf), "%d", mode);
   newmode = strtol (buf, NULL, 8);
 
   if (chmod (file, newmode) == 0) 
     {
-      request->logging_function (gftp_logging_misc, request->user_data, 
+      request->logging_function (gftp_logging_misc, request, 
                                  _("Successfully changed mode of %s to %d\n"),
                                  file, mode);
       return (0);
     }
   else 
     {
-      request->logging_function (gftp_logging_error, request->user_data, 
+      request->logging_function (gftp_logging_error, request, 
                           _("Error: Could not change mode of %s to %d: %s\n"),
                           file, mode, g_strerror (errno));
-      return (-1);
+      return (GFTP_ERETRYABLE);
     }
 }
 
@@ -628,12 +608,12 @@ local_set_file_time (gftp_request * request, const char *file,
 {
   struct utimbuf time_buf;
 
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, -2);
-  g_return_val_if_fail (file != NULL, -2);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (file != NULL, GFTP_EFATAL);
 
   time_buf.modtime = time_buf.actime = datetime;
-  return (utime (file, &time_buf));
+  return (utime (file, &time_buf) == 0 ? 0 : GFTP_ERETRYABLE);
 }
 
 
@@ -651,17 +631,26 @@ hash_function (gconstpointer key)
 }
 
 
-void
+void 
+local_register_module (void)
+{
+}
+
+
+int
 local_init (gftp_request * request)
 {
   local_protocol_data *lpd;
 
-  g_return_if_fail (request != NULL);
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
   request->protonum = GFTP_LOCAL_NUM;
   request->init = local_init;
   request->destroy = local_destroy;
+  request->read_function = gftp_fd_read;
+  request->write_function = gftp_fd_write;
   request->connect = local_connect;
+  request->post_connect = NULL;
   request->disconnect = local_disconnect;
   request->get_file = local_get_file;
   request->put_file = local_put_file;
@@ -672,7 +661,6 @@ local_init (gftp_request * request)
   request->abort_transfer = local_end_transfer; /* NOTE: uses end_transfer */
   request->list_files = local_list_files;
   request->get_next_file = local_get_next_file;
-  request->set_data_type = NULL;
   request->get_file_size = local_get_file_size;
   request->chdir = local_chdir;
   request->rmdir = local_rmdir;
@@ -686,13 +674,11 @@ local_init (gftp_request * request)
   request->set_config_options = NULL;
   request->swap_socks = NULL;
   request->url_prefix = "file";
-  request->protocol_name = "Local";
   request->need_hostport = 0;
   request->need_userpass = 0;
   request->use_cache = 0;
   request->use_threads = 0;
   request->always_connected = 1;
-  gftp_set_config_options (request);
 
   lpd = g_malloc0 (sizeof (*lpd));
   request->protocol_data = lpd;
@@ -702,5 +688,7 @@ local_init (gftp_request * request)
   if (request->hostname != NULL)
     g_free (request->hostname);
   request->hostname = g_strdup (_("local filesystem"));
+
+  return (gftp_set_config_options (request));
 }
 
