@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  misc.c - general purpose routines                                        */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -17,30 +17,42 @@
 /*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA      */
 /*****************************************************************************/
 
+static const char cvsid[] = "$Id: misc.c,v 1.46 2003/08/18 00:57:57 masneyb Exp $";
+
 #include "gftp.h"
 #include "options.h"
-static const char cvsid[] = "$Id: misc.c,v 1.16 2002/12/06 02:28:19 masneyb Exp $";
 
+/* FIXME - this isn't right for all locales. Use glib's printf instead */
 char *
 insert_commas (off_t number, char *dest_str, size_t dest_len)
 {
   char *frompos, *topos, src[50], *dest;
   int len, num, rem, i;
 
+#if defined (_LARGEFILE_SOURCE)
+  g_snprintf (src, sizeof (src), "%lld", number);
+#else
+  g_snprintf (src, sizeof (src), "%ld", number);
+#endif
+
   if (dest_str != NULL)
     *dest_str = '\0';
-  len = (number > 0 ? log10 (number) : 0) + 2;
 
-  if (len <= 0) 
+  len = strlen (src);
+  if (len == 0)
     {
       if (dest_str != NULL)
-        strncpy (dest_str, "0", dest_len);
+        {
+          strncpy (dest_str, "0", dest_len);
+          dest_str[dest_len - 1] = '\0';
+        }
       else
         dest_str = g_strdup ("0");
       return (dest_str);
     }
 
   len += len / 3;
+
   if (dest_str != NULL && len > dest_len)
     {
       
@@ -55,12 +67,6 @@ insert_commas (off_t number, char *dest_str, size_t dest_len)
   else
     dest = dest_str;
 
-#if defined (_LARGEFILE_SOURCE)
-  g_snprintf (src, sizeof (src), "%lld", number);
-#else
-  g_snprintf (src, sizeof (src), "%ld", number);
-#endif
-
   num = strlen (src) / 3 - 1;
   rem = strlen (src) % 3;
   frompos = src;
@@ -87,31 +93,6 @@ insert_commas (off_t number, char *dest_str, size_t dest_len)
 }
 
 
-long
-file_countlf (int filefd, long endpos)
-{
-  char tempstr[255];
-  long num, mypos;
-  ssize_t n;
-  int i;
-
-  mypos = num = 0;
-  lseek (filefd, 0, SEEK_SET);
-  while ((n = read (filefd, tempstr, sizeof (tempstr))) > 0)
-    {
-      for (i = 0; i < n; i++)
-	{
-	  if ((tempstr[i] == '\n') && (i > 0) && (tempstr[i - 1] != '\r')
-	      && (endpos == 0 || mypos + i <= endpos))
-	    ++num;
-	}
-      mypos += n;
-    }
-  lseek (filefd, 0, SEEK_SET);
-  return (num);
-}
-
-
 char *
 alltrim (char *str)
 {
@@ -119,7 +100,7 @@ alltrim (char *str)
   int diff;
 
   pos = str + strlen (str) - 1;
-  while (pos >= str && *pos == ' ')
+  while (pos >= str && (*pos == ' ' || *pos == '\t'))
     *pos-- = '\0';
 
   pos = str;
@@ -147,8 +128,7 @@ expand_path (const char *src)
   struct passwd *pw;
 
   pw = NULL;
-  str = g_malloc (strlen (src) + 1);
-  strcpy (str, src);
+  str = g_strdup (src);
 
   if (*str == '~')
     {
@@ -173,10 +153,13 @@ expand_path (const char *src)
       pos++;
       while (*pos == '/')
         pos++;
+
       if ((endpos = strchr (pos, '/')) == NULL)
 	endpos = pos + strlen (pos);
+
       tempchar = *endpos;
       *endpos = '\0';
+
       if (strcmp (pos, "..") == 0)
 	{
 	  *(pos - 1) = '\0';
@@ -194,18 +177,28 @@ expand_path (const char *src)
 	      newstr = tempstr;
 	    }
 	}
+
       *endpos = tempchar;
       if (*endpos == '\0')
 	break;
+
       endpos = pos + 1;
     }
 
+  if (endpos != NULL && *endpos != '\0' && newstr == NULL)
+    {
+      if (strcmp (endpos, "..") == 0)
+        newstr = g_malloc0 (1);
+      else
+        newstr = g_strdup (endpos);
+    }
+
   if (newstr == NULL || *newstr == '\0')
     {
       if (newstr != NULL)
 	g_free (newstr);
-      newstr = g_malloc0 (2);
-      *newstr = '/';
+
+      newstr = g_strdup ("/");
     }
 
   g_free (str);
@@ -213,12 +206,9 @@ expand_path (const char *src)
   if (pw != NULL)
     {
       if ((pos = strchr (newstr, '/')) == NULL)
-	{
-	  str = g_malloc (strlen (pw->pw_dir) + 1);
-	  strcpy (str, pw->pw_dir);
-	}
+	str = g_strdup (pw->pw_dir);
       else
-	str = g_strconcat (pw->pw_dir, pos, NULL);
+	str = gftp_build_path (pw->pw_dir, pos, NULL);
 
       g_free (newstr);
       newstr = str;
@@ -232,6 +222,7 @@ void
 remove_double_slashes (char *string)
 {
   char *newpos, *oldpos;
+  size_t len;
 
   oldpos = newpos = string;
   while (*oldpos != '\0')
@@ -243,8 +234,10 @@ remove_double_slashes (char *string)
 	oldpos++;
     }
   *newpos = '\0';
-  if (string[strlen (string) - 1] == '/')
-    string[strlen (string) - 1] = '\0';
+
+  len = strlen (string);
+  if (string[len - 1] == '/')
+    string[len - 1] = '\0';
 }
 
 
@@ -263,14 +256,14 @@ copyfile (char *source, char *dest)
   char buf[8192];
   ssize_t n;
 
-  if ((srcfd = open (source, O_RDONLY)) == -1)
+  if ((srcfd = gftp_fd_open (NULL, source, O_RDONLY, 0)) == -1)
     {
       printf (_("Error: Cannot open local file %s: %s\n"),
               source, g_strerror (errno));
       exit (1);
     }
 
-  if ((destfd = open (dest, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR)) == -1)
+  if ((destfd = gftp_fd_open (NULL, dest, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR)) == -1)
     {
       printf (_("Error: Cannot open local file %s: %s\n"),
               dest, g_strerror (errno));
@@ -294,13 +287,23 @@ copyfile (char *source, char *dest)
       exit (1);
     }
 
-  close (srcfd);
-  close (destfd);
+  if (close (srcfd) == -1)
+    {
+      printf (_("Error closing file descriptor: %s\n"), g_strerror (errno));
+      exit (1);
+    }
+
+  if (close (destfd) == -1)
+    {
+      printf (_("Error closing file descriptor: %s\n"), g_strerror (errno));
+      exit (1);
+    }
 
   return (1);
 }
 
 
+/* FIXME - is there a replacement for this */
 int
 gftp_match_filespec (char *filename, char *filespec)
 {
@@ -346,16 +349,86 @@ gftp_match_filespec (char *filename, char *filespec)
 }
 
 
+static void
+gftp_info (void)
+{
+  int i;
+
+  printf ("%s\n", gftp_version);
+
+#ifdef _LARGEFILE_SOURCE
+  printf ("#define _LARGEFILE_SOURCE\n");
+#endif
+
+#ifdef _FILE_OFFSET_BITS
+  printf ("#define _FILE_OFFSET_BITS %d\n", _FILE_OFFSET_BITS);
+#endif
+
+  printf ("sizeof (off_t) = %d\n", sizeof (off_t));
+
+#ifdef HAVE_GETADDRINFO
+  printf ("#define HAVE_GETADDRINFO\n");
+#endif
+
+#ifdef HAVE_GAI_STRERROR
+  printf ("#define HAVE_GAI_STRERROR\n");
+#endif
+
+#ifdef HAVE_GETDTABLESIZE
+  printf ("#define HAVE_GETDTABLESIZE\n");
+#endif
+
+#ifdef G_HAVE_GINT64
+  printf ("#define G_HAVE_GINT64\n");
+#endif
+
+#ifdef HAVE_LIBREADLINE
+  printf ("#define HAVE_LIBREADLINE\n");
+#endif
+
+#ifdef ENABLE_NLS
+  printf ("#define ENABLE_NLS\n");
+#endif
+
+#ifdef HAVE_GETTEXT
+  printf ("#define HAVE_GETTEXT\n");
+#endif
+
+  printf ("glib version: %d.%d.%d\n", GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION,
+          GLIB_MICRO_VERSION);
+
+  printf ("PTY implementation: %s\n", gftp_get_pty_impl ());
+
+#ifdef USE_SSL
+  printf ("OpenSSL version: 0x%lx\n", OPENSSL_VERSION_NUMBER);
+#endif
+
+  printf ("Enabled protocols: ");
+  for (i=0; gftp_protocols[i].name != NULL; i++)
+    {
+      printf ("%s ", gftp_protocols[i].name);
+    }
+  printf ("\n");
+}
+
+
 int
 gftp_parse_command_line (int *argc, char ***argv)
 {
   if (*argc > 1)
     {
-      if (strcmp (argv[0][1], "--help") == 0 || strcmp (argv[0][1], "-h") == 0)
+      if (strcmp (argv[0][1], "--help") == 0 || 
+          strcmp (argv[0][1], "-h") == 0)
 	gftp_usage ();
-      else if (strcmp (argv[0][1], "--version") == 0 || strcmp (argv[0][1], "-v") == 0)
+      else if (strcmp (argv[0][1], "--version") == 0 || 
+               strcmp (argv[0][1], "-v") == 0)
 	{
-	  printf ("%s\n", version);
+	  printf ("%s\n", gftp_version);
+	  exit (0);
+	}
+      else if (strcmp (argv[0][1], "--info") == 0)
+	{
+          gftp_info ();
 	  exit (0);
 	}
     }
@@ -366,44 +439,11 @@ gftp_parse_command_line (int *argc, char ***argv)
 void
 gftp_usage (void)
 {
-  printf (_("usage: gftp [[ftp://][user:[pass]@]ftp-site[:port][/directory]]\n"));
+  printf (_("usage: gftp [[protocol://][user[:pass]@]site[:port][/directory]]\n"));
   exit (0);
 }
 
 
-char *
-get_xpm_path (char *filename, int quit_on_err)
-{
-  char *tempstr, *exfile;
-
-  tempstr = g_strconcat (BASE_CONF_DIR, "/", filename, NULL);
-  exfile = expand_path (tempstr);
-  g_free (tempstr);
-  if (access (exfile, F_OK) != 0)
-    {
-      g_free (exfile);
-      tempstr = g_strconcat (SHARE_DIR, "/", filename, NULL);
-      exfile = expand_path (tempstr);
-      g_free (tempstr);
-      if (access (exfile, F_OK) != 0)
-	{
-	  g_free (exfile);
-	  exfile = g_strconcat ("/usr/share/icons/", filename, NULL);
-	  if (access (exfile, F_OK) != 0)
-	    {
-	      g_free (exfile);
-	      if (!quit_on_err)
-		return (NULL);
-	      printf (_("gFTP Error: Cannot find file %s in %s or %s\n"),
-		      filename, SHARE_DIR, BASE_CONF_DIR);
-	      exit (-1);
-	    }
-	}
-    }
-  return (exfile);
-}
-
-
 gint
 string_hash_compare (gconstpointer path1, gconstpointer path2)
 {
@@ -418,7 +458,7 @@ string_hash_function (gconstpointer key)
   int i;
 
   ret = 0;
-  for (i=0; ((char *) key)[0] != '\0' && i < 3; i++)
+  for (i=0; ((char *) key)[i] != '\0' && i < 3; i++)
     ret += ((char *) key)[i];
 
   return (ret);
@@ -447,6 +487,8 @@ free_fdata (gftp_file * fle)
 {
   if (fle->file)
     g_free (fle->file);
+  if (fle->utf8_file)
+    g_free (fle->utf8_file);
   if (fle->user)
     g_free (fle->user);
   if (fle->group)
@@ -470,34 +512,23 @@ copy_fdata (gftp_file * fle)
   memcpy (newfle, fle, sizeof (*newfle));
 
   if (fle->file)
-    {
-      newfle->file = g_malloc (strlen (fle->file) + 1);
-      strcpy (newfle->file, fle->file);
-    }
+    newfle->file = g_strdup (fle->file);
+
+  if (fle->utf8_file)
+    newfle->utf8_file = g_strdup (fle->utf8_file);
 
   if (fle->user)
-    {
-      newfle->user = g_malloc (strlen (fle->user) + 1);
-      strcpy (newfle->user, fle->user);
-    }
+    newfle->user = g_strdup (fle->user);
 
   if (fle->group)
-    {
-      newfle->group = g_malloc (strlen (fle->group) + 1);
-      strcpy (newfle->group, fle->group);
-    }
+    newfle->group = g_strdup (fle->group);
 
   if (fle->attribs)
-    {
-      newfle->attribs = g_malloc (strlen (fle->attribs) + 1);
-      strcpy (newfle->attribs, fle->attribs);
-    }
+    newfle->attribs = g_strdup (fle->attribs);
 
   if (fle->destfile)
-    {
-      newfle->destfile = g_malloc (strlen (fle->destfile) + 1);
-      strcpy (newfle->destfile, fle->destfile);
-    }
+    newfle->destfile = g_strdup (fle->destfile);
+
   return (newfle);
 }
 
@@ -510,7 +541,7 @@ compare_request (gftp_request * request1, gftp_request * request2,
   int i, ret;
 
   ret = 1;
-  if (strcmp (request1->protocol_name, request2->protocol_name) == 0 &&
+  if (request1->protonum == request2->protonum &&
       request1->port == request2->port)
     {
       strarr[0][0] = request1->hostname;
@@ -548,13 +579,31 @@ compare_request (gftp_request * request1, gftp_request * request2,
 }
 
 
+gftp_transfer *
+gftp_tdata_new (void)
+{
+#if GLIB_MAJOR_VERSION == 1
+  static GStaticMutex init_mutex = G_STATIC_MUTEX_INIT;
+#endif
+  gftp_transfer * tdata;
+
+  tdata = g_malloc0 (sizeof (*tdata));
+
+#if GLIB_MAJOR_VERSION == 1
+  tdata->statmutex = init_mutex;
+  tdata->structmutex = init_mutex;
+#else
+  g_static_mutex_init (&tdata->statmutex);
+  g_static_mutex_init (&tdata->structmutex);
+#endif
+
+  return (tdata);
+}
+
+
 void
 free_tdata (gftp_transfer * tdata)
 {
-  if (tdata->statmutex)
-    g_free (tdata->statmutex);
-  if (tdata->structmutex)
-    g_free (tdata->structmutex);
   if (tdata->fromreq != NULL)
     gftp_request_destroy (tdata->fromreq, 1);
   if (tdata->toreq != NULL)
@@ -565,7 +614,7 @@ free_tdata (gftp_transfer * tdata)
 
 
 gftp_request * 
-copy_request (gftp_request * req)
+copy_request (gftp_request * req, int copy_local_options)
 {
   gftp_request * newreq;
 
@@ -582,421 +631,27 @@ copy_request (gftp_request * req)
   if (req->directory)
     newreq->directory = g_strdup (req->directory);
   newreq->port = req->port;
-  newreq->data_type = req->data_type;
   newreq->use_proxy = req->use_proxy;
-  gftp_set_proxy_config (newreq, req->proxy_config);
-  newreq->transfer_type = req->transfer_type;
-  newreq->network_timeout = req->network_timeout;
-  newreq->retries = req->retries;
-  newreq->sleep_time = req->sleep_time;
-  newreq->passive_transfer = req->passive_transfer;
-  newreq->maxkbs = req->maxkbs;
   newreq->logging_function = req->logging_function;
+  newreq->free_hostp = 0;
+  newreq->hostp = req->hostp;
 
-  if (req->sftpserv_path != NULL)
-    newreq->sftpserv_path = g_strdup (req->sftpserv_path);
+  if (copy_local_options)
+    gftp_copy_local_options (&newreq->local_options_vars, 
+                             &newreq->local_options_hash,
+                             req->local_options_vars,
+                             req->num_local_options_vars);
 
-  req->init (newreq);
+  if (req->init (newreq) < 0)
+    {
+      gftp_request_destroy (newreq, 1);
+      return (NULL);
+    }
 
   return (newreq);
 }
 
 
-int
-ptym_open (char *pts_name)
-{
-  int fd;
-
-#ifdef __sgi
-  char *tempstr;
-
-  if ((tempstr = _getpty (&fd, O_RDWR, 0600, 0)) == NULL)
-    return (-1);
-
-  strcpy (pts_name, tempstr);
-  return (fd);
-
-#else /* !__sgi */
-
-#ifdef SYSV
-
-  char *tempstr;
-
-  strcpy (pts_name, "/dev/ptmx");
-  if ((fd = open (pts_name, O_RDWR)) < 0)
-    return (-1);
-
-  if (grantpt (fd) < 0)
-    {
-      close (fd);
-      return (-1);
-    }
-
-  if (unlockpt (fd) < 0)
-    {
-      close (fd);
-      return (-1);
-    }
-
-  if ((tempstr = ptsname (fd)) == NULL)
-    {
-      close (fd);
-      return (-1);
-    }
-
-  strcpy (pts_name, tempstr);
-  return (fd);
-
-#else /* !SYSV */
-
-  char *pos1, *pos2;
-
-  strcpy (pts_name, "/dev/ptyXY");
-  for (pos1 = "pqrstuvwxyzPQRST"; *pos1 != '\0'; pos1++) 
-    {
-      pts_name[8] = *pos1;
-      for (pos2 = "0123456789abcdef"; *pos2 != '\0'; pos2++)
-        {
-          pts_name[9] = *pos2;
-          if ((fd = open (pts_name, O_RDWR)) < 0)
-            continue;
-          pts_name[5] = 't';
-          return (fd);
-        }
-    }
-  return (-1);
-
-#endif
-
-#endif
-
-}
-
-
-int
-ptys_open (int fdm, char *pts_name)
-{
-  int fds;
-
-#if !defined (SYSV) && !defined (__sgi)
-
-  chmod (pts_name, S_IRUSR | S_IWUSR);
-  chown (pts_name, getuid (), -1);
-
-#endif
-
-  if ((fds = open (pts_name, O_RDWR)) < 0)
-    {
-      close (fdm);
-      return (-1);
-    }
-
-#ifdef SYSV
-
-  if (ioctl (fds, I_PUSH, "ptem") < 0)
-    {
-      close (fdm);
-      close (fds);
-      return (-1);
-    }
-
-  if (ioctl (fds, I_PUSH, "ldterm") < 0)
-    {
-      close (fdm);
-      close (fds);
-      return (-1);
-    }
-
-  if (ioctl (fds, I_PUSH, "ttcompat") < 0)
-    {
-      close (fdm);
-      close (fds);
-      return (-1);
-    }
-
-#endif
-
-#if !defined(SYSV) && !defined (__sgi) && defined(TIOCSCTTY) && !defined(CIBAUD)
-
-  if (ioctl (fds, TIOCSCTTY, (char *) 0) < 0)
-    {
-      close (fdm);
-      return (-1);
-    }
-
-#endif
-
-  return (fds);
-}
-
-
-int
-tty_raw (int fd)
-{
-  struct termios buf;
-
-  if (tcgetattr (fd, &buf) < 0)
-    return (-1);
-
-  buf.c_iflag |= IGNPAR;
-  buf.c_iflag &= ~(ICRNL | ISTRIP | IXON | IGNCR | IXANY | IXOFF | INLCR);
-  buf.c_lflag &= ~(ECHO | ICANON | ISIG | ECHOE | ECHOK | ECHONL);
-#ifdef IEXTEN
-  buf.c_lflag &= ~(IEXTEN);
-#endif
-
-  buf.c_oflag &= ~(OPOST);
-  buf.c_cc[VMIN] = 1;
-  buf.c_cc[VTIME] = 0;
-
-  if (tcsetattr (fd, TCSADRAIN, &buf) < 0)
-    return (-1);
-  return (0);
-}
-
-
-/* We have the caller send us a pointer to a string so we can write the port
-   into it. It makes it easier so we don't have to worry about freeing it 
-   later on, the caller can just send us an auto variable, The string
-   should be at least 6 chars. I know this is messy... */
-
-char **
-make_ssh_exec_args (gftp_request * request, char *execname, 
-                    int use_sftp_subsys, char *portstring)
-{
-  char **args, *oldstr, *tempstr;
-  struct servent serv_struct;
-  int i, j;
-
-  args = g_malloc (sizeof (char *) * (num_ssh_extra_params + 15));
-
-  args[0] = ssh_prog_name != NULL && *ssh_prog_name != '\0' ? 
-            ssh_prog_name : "ssh";
-  i = 1;
-  tempstr = g_strdup (args[0]);
-
-  if (ssh_extra_params_list != NULL)
-    {
-      for (j=0; ssh_extra_params_list[j] != NULL; j++)
-        {
-          oldstr = tempstr;
-          args[i++] = ssh_extra_params_list[j];
-          tempstr = g_strconcat (oldstr, " ", ssh_extra_params_list[j], NULL);
-          g_free (oldstr);
-        }
-    }
-
-  oldstr = tempstr;
-  tempstr = g_strconcat (oldstr, " -e none", NULL);
-  g_free (oldstr);
-  args[i++] = "-e";
-  args[i++] = "none";
-
-  if (request->username && *request->username != '\0')
-    {
-      oldstr = tempstr;
-      tempstr = g_strconcat (oldstr, " -l ", request->username, NULL);
-      g_free (oldstr);
-      args[i++] = "-l";
-      args[i++] = request->username;
-    }
-
-  if (request->port != 0)
-    {
-      g_snprintf (portstring, 6, "%d", request->port);
-      oldstr = tempstr;
-      tempstr = g_strconcat (oldstr, " -p ", portstring, NULL);
-      g_free (oldstr);
-      args[i++] = "-p";
-      args[i++] = portstring;
-    }
-  else
-    {
-      if (!r_getservbyname ("ssh", "tcp", &serv_struct, NULL))
-        request->port = 22;
-      else
-        request->port = ntohs (serv_struct.s_port);
-    }
-
-  if (use_sftp_subsys)
-    {
-      oldstr = tempstr;
-      tempstr = g_strconcat (oldstr, " ", request->hostname, " -s sftp", NULL);
-      g_free (oldstr);
-      args[i++] = request->hostname;
-      args[i++] = "-s";
-      args[i++] = "sftp";
-      args[i] = NULL;
-    }
-  else
-    {
-      oldstr = tempstr;
-      tempstr = g_strconcat (oldstr, " ", request->hostname, " \"", execname, 
-                             "\"", NULL);
-      g_free (oldstr);
-      args[i++] = request->hostname;
-      args[i++] = execname;
-      args[i] = NULL;
-    }
-
-  request->logging_function (gftp_logging_misc, request->user_data, 
-                             _("Running program %s\n"), tempstr);
-  g_free (tempstr);
-  return (args);
-}
-
-#define SSH_LOGIN_BUFSIZE	200
-#define SSH_ERROR_BADPASS	-1
-#define SSH_ERROR_QUESTION	-2
-#define SSH_WARNING 		-3
-
-char *
-ssh_start_login_sequence (gftp_request * request, int fd)
-{
-  char *tempstr, *pwstr, *tmppos;
-  size_t rem, len, diff, lastdiff, key_pos;
-  int wrotepw, ok;
-  ssize_t rd;
-
-  rem = len = SSH_LOGIN_BUFSIZE;
-  tempstr = g_malloc0 (len + 1);
-  key_pos = diff = lastdiff = 0;
-  wrotepw = 0;
-  ok = 1;
-
-  if (gftp_set_sockblocking (request, fd, 1) == -1)
-    return (NULL);
-
-  pwstr = g_strconcat (request->password, "\n", NULL);
-
-  errno = 0;
-  while (1)
-    {
-      if ((rd = gftp_read (request, tempstr + diff, rem - 1, fd)) <= 0)
-        {
-          ok = 0;
-          break;
-        }
-      rem -= rd;
-      diff += rd;
-      tempstr[diff] = '\0'; 
-
-      if (diff > 11 && strcmp (tempstr + diff - 10, "password: ") == 0)
-        {
-          if (wrotepw)
-            {
-              ok = SSH_ERROR_BADPASS;
-              break;
-            }
-
-          if (strstr (tempstr, "WARNING") != NULL ||
-              strstr (tempstr, _("WARNING")) != NULL)
-            {
-              ok = SSH_WARNING;
-              break;
-            }
-              
-          wrotepw = 1;
-          if (gftp_write (request, pwstr, strlen (pwstr), fd) < 0)
-            {
-              ok = 0;
-              break;
-            }
-        }
-      else if (diff > 2 && strcmp (tempstr + diff - 2, ": ") == 0 &&
-               ((tmppos = strstr (tempstr + key_pos, "Enter passphrase for RSA key")) != NULL ||
-                ((tmppos = strstr (tempstr + key_pos, "Enter passphrase for key '")) != NULL)))
-        {
-          key_pos = diff;
-          if (wrotepw)
-            {
-              ok = SSH_ERROR_BADPASS;
-              break;
-            }
-
-          if (strstr (tempstr, "WARNING") != NULL ||
-              strstr (tempstr, _("WARNING")) != NULL)
-            {
-              ok = SSH_WARNING;
-              break;
-            }
-
-          wrotepw = 1;
-          if (gftp_write (request, pwstr, strlen (pwstr), fd) < 0)
-            {
-              ok = 0;
-              break;
-            }
-        }
-      else if (diff > 10 && strcmp (tempstr + diff - 10, "(yes/no)? ") == 0)
-        {
-          ok = SSH_ERROR_QUESTION;
-          break;
-        }
-      else if (diff >= 5 && strcmp (tempstr + diff - 5, "xsftp") == 0)
-        break;
-      else if (rem <= 1)
-        {
-          request->logging_function (gftp_logging_recv, request->user_data,
-                                     "%s", tempstr + lastdiff);
-          len += SSH_LOGIN_BUFSIZE;
-          rem += SSH_LOGIN_BUFSIZE;
-          lastdiff = diff;
-          tempstr = g_realloc (tempstr, len);
-          continue;
-        }
-    }
-
-  g_free (pwstr);
-
-  if (*(tempstr + lastdiff) != '\0')
-    request->logging_function (gftp_logging_recv, request->user_data,
-                               "%s\n", tempstr + lastdiff);
-
-  if (ok <= 0)
-    {
-      if (ok == SSH_ERROR_BADPASS)
-        request->logging_function (gftp_logging_error, request->user_data,
-                               _("Error: An incorrect password was entered\n"));
-      else if (ok == SSH_ERROR_QUESTION)
-        request->logging_function (gftp_logging_error, request->user_data,
-                               _("Please connect to this host with the command line SSH utility and answer this question appropriately.\n"));
-      else if (ok == SSH_WARNING)
-        request->logging_function (gftp_logging_error, request->user_data,
-                                   _("Please correct the above warning to connect to this host.\n"));
-
-      g_free (tempstr);
-      return (NULL);
-    }
- 
-  return (tempstr);
-}
-
-
-#ifdef G_HAVE_GINT64
-
-gint64
-hton64 (gint64 val)
-{
-  gint64 num;
-  char *pos;
-
-  num = 0;
-  pos = (char *) &num;
-  pos[0] = (val >> 56) & 0xff;
-  pos[1] = (val >> 48) & 0xff;
-  pos[2] = (val >> 40) & 0xff;
-  pos[3] = (val >> 32) & 0xff;
-  pos[4] = (val >> 24) & 0xff;
-  pos[5] = (val >> 16) & 0xff;
-  pos[6] = (val >> 8) & 0xff;
-  pos[7] = val & 0xff;
-  return (num);
-}
-
-#endif
-
-
 static gint
 gftp_file_sort_function_as (gconstpointer a, gconstpointer b)
 {
@@ -1017,11 +672,7 @@ gftp_file_sort_function_ds (gconstpointer a, gconstpointer b)
   f1 = a;
   f2 = b;
   ret = strcmp (f1->file, f2->file);
-  if (ret < 0)
-    ret = 1;
-  else if (ret > 0)
-    ret = -1;
-  return (ret);
+  return (ret * -1);
 }
 
 
@@ -1045,11 +696,7 @@ gftp_user_sort_function_ds (gconstpointer a, gconstpointer b)
   f1 = a;
   f2 = b;
   ret = strcmp (f1->user, f2->user);
-  if (ret < 0)
-    ret = 1;
-  else if (ret > 0)
-    ret = -1;
-  return (ret);
+  return (ret * -1);
 }
 
 
@@ -1073,11 +720,7 @@ gftp_group_sort_function_ds (gconstpointer a, gconstpointer b)
   f1 = a;
   f2 = b;
   ret = strcmp (f1->group, f2->group);
-  if (ret < 0)
-    ret = 1;
-  else if (ret > 0)
-    ret = -1;
-  return (ret);
+  return (ret * -1);
 }
 
 
@@ -1101,11 +744,7 @@ gftp_attribs_sort_function_ds (gconstpointer a, gconstpointer b)
   f1 = a;
   f2 = b;
   ret = strcmp (f1->attribs, f2->attribs);
-  if (ret < 0)
-    ret = 1;
-  else if (ret > 0)
-    ret = -1;
-  return (ret);
+  return (ret * -1);
 }
 
 
@@ -1179,6 +818,7 @@ gftp_sort_filelist (GList * filelist, int column, int asds)
   GList * files, * dirs, * dotdot, * tempitem, * insitem;
   GCompareFunc sortfunc;
   gftp_file * tempfle;
+  int sort_dirs_first;
 
   files = dirs = dotdot = NULL;
 
@@ -1193,9 +833,15 @@ gftp_sort_filelist (GList * filelist, int column, int asds)
                 gftp_group_sort_function_as : gftp_group_sort_function_ds;
   else if (column == GFTP_SORT_COL_DATETIME)
     sortfunc = asds ?
-                gftp_datetime_sort_function_as : gftp_datetime_sort_function_ds;  else /* GFTP_SORT_COL_ATTRIBS */
+                gftp_datetime_sort_function_as : gftp_datetime_sort_function_ds;
+  else if (column == GFTP_SORT_COL_ATTRIBS)
     sortfunc = asds ? 
                 gftp_attribs_sort_function_as : gftp_attribs_sort_function_ds;
+  else /* Don't sort */
+   return (filelist);
+
+  sort_dirs_first = 1;
+  gftp_lookup_global_option ("sort_dirs_first", &sort_dirs_first);
 
   for (tempitem = filelist; tempitem != NULL; )
     {
@@ -1241,3 +887,368 @@ gftp_sort_filelist (GList * filelist, int column, int asds)
   return (filelist);
 }
 
+
+mode_t
+gftp_parse_attribs (char *attribs)
+{
+  mode_t mode;
+  int cur;
+
+  cur = 0;
+  if (attribs[1] == 'r')
+    cur += 4;
+  if (attribs[2] == 'w')
+    cur += 2;
+  if (attribs[3] == 'x' ||
+      attribs[3] == 's')
+    cur += 1;
+  mode = cur;
+
+  cur = 0;
+  if (attribs[4] == 'r')
+    cur += 4;
+  if (attribs[5] == 'w')
+    cur += 2;
+  if (attribs[6] == 'x' ||
+      attribs[6] == 's')
+    cur += 1;
+  mode = (mode * 10) + cur;
+
+  cur = 0;
+  if (attribs[7] == 'r')
+    cur += 4;
+  if (attribs[8] == 'w')
+    cur += 2;
+  if (attribs[9] == 'x' ||
+      attribs[9] == 's')
+    cur += 1;
+  mode = (mode * 10) + cur;
+
+  return (mode);
+}
+
+
+char *
+gftp_gen_ls_string (gftp_file * fle, char *file_prefixstr, char *file_suffixstr)
+{
+  char *tempstr1, *tempstr2, *ret, tstr[50];
+  struct tm *lt;
+  time_t t;
+
+  lt = localtime (&fle->datetime);
+
+  tempstr1 = g_strdup_printf ("%10s %8s %8s", fle->attribs, fle->user,
+                              fle->group);
+
+  if (fle->attribs && (*fle->attribs == 'b' || *fle->attribs == 'c'))
+    tempstr2 = g_strdup_printf ("%d, %d", major (fle->size), minor (fle->size));
+  else
+    {
+#if defined (_LARGEFILE_SOURCE)
+      tempstr2 = g_strdup_printf ("%11lld", fle->size);
+#else
+      tempstr2 = g_strdup_printf ("%11ld", fle->size);
+#endif
+    }
+
+  time (&t);
+
+  if (fle->datetime > t || t - 3600*24*90 > fle->datetime)
+    strftime (tstr, sizeof (tstr), "%b %d  %Y", lt);
+  else
+    strftime (tstr, sizeof (tstr), "%b %d %H:%M", lt);
+
+  if (file_prefixstr == NULL)
+    file_prefixstr = "";
+  if (file_suffixstr == NULL)
+    file_suffixstr = "";
+
+  ret = g_strdup_printf ("%s %s %s %s%s%s", tempstr1, tempstr2, tstr, 
+                         file_prefixstr, 
+                         fle->utf8_file != NULL ? fle->utf8_file : fle->file,
+                         file_suffixstr);
+
+  g_free (tempstr1);
+  g_free (tempstr2);
+
+  return (ret);
+}
+
+
+#if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
+
+struct hostent *
+r_gethostbyname (const char *name, struct hostent *result_buf, int *h_errnop)
+{
+  static GStaticMutex hostfunclock = G_STATIC_MUTEX_INIT; 
+  struct hostent *hent;
+
+  if (g_thread_supported ())
+    g_static_mutex_lock (&hostfunclock);
+
+  if ((hent = gethostbyname (name)) == NULL)
+    {
+      if (h_errnop)
+        *h_errnop = h_errno;
+    }
+  else
+    {
+      *result_buf = *hent;
+      hent = result_buf;
+    }
+
+  if (g_thread_supported ())
+    g_static_mutex_unlock (&hostfunclock);
+
+  return (hent);
+}
+
+#endif /* !HAVE_GETADDRINFO */
+
+struct servent *
+r_getservbyname (const char *name, const char *proto,
+                 struct servent *result_buf, int *h_errnop)
+{
+  static GStaticMutex servfunclock = G_STATIC_MUTEX_INIT;
+  struct servent *sent;
+
+  if (g_thread_supported ())
+    g_static_mutex_lock (&servfunclock);
+
+  if ((sent = getservbyname (name, proto)) == NULL)
+    {
+      if (h_errnop)
+        *h_errnop = h_errno;
+    }
+  else
+    {
+      *result_buf = *sent;
+      sent = result_buf;
+    }
+
+  if (g_thread_supported ())
+    g_static_mutex_unlock (&servfunclock);
+  return (sent);
+}
+
+
+char *
+base64_encode (char *str)
+{
+
+/* The standard to Base64 encoding can be found in RFC2045 */
+
+  char *newstr, *newpos, *fillpos, *pos;
+  unsigned char table[64], encode[3];
+  int i, num;
+
+  for (i = 0; i < 26; i++)
+    {
+      table[i] = 'A' + i;
+      table[i + 26] = 'a' + i;
+    }
+
+  for (i = 0; i < 10; i++)
+    table[i + 52] = '0' + i;
+
+  table[62] = '+';
+  table[63] = '/';
+
+  num = strlen (str) / 3;
+  if (strlen (str) % 3 > 0)
+    num++;
+  newstr = g_malloc (num * 4 + 1);
+  newstr[num * 4] = '\0';
+  newpos = newstr;
+
+  pos = str;
+  while (*pos != '\0')
+    {
+      memset (encode, 0, sizeof (encode));
+      for (i = 0; i < 3 && *pos != '\0'; i++)
+	encode[i] = *pos++;
+
+      fillpos = newpos;
+      *newpos++ = table[encode[0] >> 2];
+      *newpos++ = table[(encode[0] & 3) << 4 | encode[1] >> 4];
+      *newpos++ = table[(encode[1] & 0xF) << 2 | encode[2] >> 6];
+      *newpos++ = table[encode[2] & 0x3F];
+      while (i < 3)
+	fillpos[++i] = '=';
+    }
+  return (newstr);
+}
+
+
+void
+gftp_free_bookmark (gftp_bookmarks_var * entry)
+{
+  if (entry->hostname)
+    g_free (entry->hostname);
+  if (entry->remote_dir)
+    g_free (entry->remote_dir);
+  if (entry->local_dir)
+    g_free (entry->local_dir);
+  if (entry->user)
+    g_free (entry->user);
+  if (entry->pass)
+    g_free (entry->pass);
+  if (entry->acct)
+    g_free (entry->acct);
+  if (entry->protocol)
+    g_free (entry->protocol);
+
+  if (entry->local_options_vars != NULL)
+    {
+      gftp_config_free_options (entry->local_options_vars,
+                                entry->local_options_hash,
+                                entry->num_local_options_vars);
+
+      entry->local_options_vars = NULL;
+      entry->local_options_hash = NULL;
+      entry->num_local_options_vars = 0;
+    }
+}
+
+
+void
+gftp_shutdown (void)
+{
+#ifdef WITH_DMALLOC
+  gftp_config_vars * cv;
+  GList * templist;
+#endif
+
+  if (gftp_logfd != NULL)
+    fclose (gftp_logfd);
+
+  gftp_clear_cache_files ();
+
+  if (gftp_configuration_changed)
+    gftp_write_config_file ();
+
+#ifdef WITH_DMALLOC
+  if (gftp_global_options_htable != NULL)
+    g_hash_table_destroy (gftp_global_options_htable);
+
+  if (gftp_config_list_htable != NULL)
+    g_hash_table_destroy (gftp_config_list_htable);
+
+  if (gftp_bookmarks_htable != NULL)
+    g_hash_table_destroy (gftp_bookmarks_htable);
+
+  for (templist = gftp_options_list; 
+       templist != NULL; 
+       templist = templist->next)
+    {
+      cv = templist->data;
+      gftp_config_free_options (cv, NULL, -1);
+    }
+
+  gftp_bookmarks_destroy (gftp_bookmarks);
+
+  dmalloc_shutdown ();
+#endif
+
+  exit (0);
+}
+
+
+GList *
+get_next_selection (GList * selection, GList ** list, int *curnum)
+{
+  gftp_file * tempfle;
+  int i, newpos;
+
+  newpos = GPOINTER_TO_INT (selection->data);
+  i = *curnum - newpos;
+
+  if (i < 0)
+    {
+      while (i != 0)
+        {
+          tempfle = (*list)->data;
+          if (tempfle->shown)
+            {
+	      ++*curnum;
+	      i++;
+            }
+	  *list = (*list)->next;
+        }     
+    }
+  else if (i > 0)
+    {
+      while (i != 0)
+        {
+          tempfle = (*list)->data;
+          if (tempfle->shown)
+            {
+	      --*curnum;
+	      i--;
+            }
+	  *list = (*list)->prev;
+        }
+    }
+
+  tempfle = (*list)->data;
+  while ((*list)->next && !tempfle->shown)
+    {
+      *list = (*list)->next;
+      tempfle = (*list)->data;
+    }
+  return (selection->next);
+}
+
+
+char *
+gftp_build_path (const char *first_element, ...) 
+{
+  const char *element;
+  size_t len, retlen;
+  int add_separator;
+  va_list args;
+  char *ret;
+
+  g_return_val_if_fail (first_element != NULL, NULL);
+
+  ret = g_strdup (first_element);
+  retlen = strlen (ret);
+
+  va_start (args, first_element);
+  for (element = va_arg (args, char *);
+       element != NULL;
+       element = va_arg (args, char *))
+    {
+      len = strlen (element);
+
+      if (len > 0 && element[len - 1] == '/')
+        add_separator = 0;
+      else
+        {
+          add_separator = 1;
+          len++;
+        }
+      
+      retlen += len;
+      ret = g_realloc (ret, retlen + 1);
+
+      if (add_separator)
+        strncat (ret, "/", retlen);
+
+      strncat (ret, element, retlen);
+    }
+
+  return (ret);
+}
+
+
+off_t
+gftp_parse_file_size (char *str)
+{
+#if defined (_LARGEFILE_SOURCE)
+  return (strtoll (str, NULL, 10));
+#else
+  return (strtol (str, NULL, 10));
+#endif
+}
+
