@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  chmod_dialog.c - the chmod dialog box                                    */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: chmod_dialog.c,v 1.8 2002/11/26 14:58:13 masneyb Exp $";
+static const char cvsid[] = "$Id: chmod_dialog.c,v 1.13 2003/08/12 01:05:02 masneyb Exp $";
 
 static GtkWidget *suid, *sgid, *sticky, *ur, *uw, *ux, *gr, *gw, *gx, *or, *ow,
                  *ox;
@@ -28,13 +28,16 @@ static int mode;
 static void *
 do_chmod_thread (void * data)
 {
+  int success, num, sj, network_timeout;
   GList * filelist, * templist;
   gftp_window_data * wdata;
-  int success, num, sj;
   gftp_file * tempfle;
 
   wdata = data;
 
+  gftp_lookup_request_option (wdata->request, "network_timeout", 
+                              &network_timeout);
+
   if (wdata->request->use_threads)
     {
       sj = sigsetjmp (jmp_environment, 1);
@@ -53,8 +56,8 @@ do_chmod_thread (void * data)
         {
           templist = get_next_selection (templist, &filelist, &num);
           tempfle = filelist->data;
-          if (wdata->request->network_timeout > 0)
-            alarm (wdata->request->network_timeout);
+          if (network_timeout > 0)
+            alarm (network_timeout);
           if (gftp_chmod (wdata->request, tempfle->file, mode) == 0)
             success = 1;
           if (!GFTP_IS_CONNECTED (wdata->request))
@@ -66,7 +69,7 @@ do_chmod_thread (void * data)
     {
       gftp_disconnect (wdata->request);
       wdata->request->logging_function (gftp_logging_error, 
-                                        wdata->request->user_data,
+                                        wdata->request,
                                         _("Operation canceled\n"));
     }
 
@@ -74,14 +77,14 @@ do_chmod_thread (void * data)
     use_jmp_environment = 0;
 
   wdata->request->stopable = 0;
-  return ((void *) success);
+  return (GINT_TO_POINTER (success));
 }
 
 
 static void
 dochmod (GtkWidget * widget, gftp_window_data * wdata)
 {
-  int cur;
+  int cur, ret;
 
   mode = 0;
   if (GTK_TOGGLE_BUTTON (suid)->active)
@@ -121,7 +124,8 @@ dochmod (GtkWidget * widget, gftp_window_data * wdata)
   if (check_reconnect (wdata) < 0)
     return;
 
-   if ((int) generic_thread (do_chmod_thread, wdata))
+  ret = GPOINTER_TO_INT (generic_thread (do_chmod_thread, wdata));
+  if (ret)
     refresh (wdata);
 }
 
@@ -180,7 +184,7 @@ chmod_dialog (gpointer data)
     {
       gdk_window_set_icon (dialog->window, NULL, gftp_icon->pixmap,
                            gftp_icon->bitmap);
-      gdk_window_set_icon_name (dialog->window, _("gFTP Icon"));
+      gdk_window_set_icon_name (dialog->window, gftp_version);
     }
 
   tempwid = gtk_label_new (_("You can now adjust the attributes of your file(s)\nNote: Not all ftp servers support the chmod feature"));
