@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  delete_dialog.c - the delete dialog                                      */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,19 +18,12 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: delete_dialog.c,v 1.11 2002/11/23 13:45:05 masneyb Exp $";
+static const char cvsid[] = "$Id: delete_dialog.c,v 1.16 2003/08/12 01:05:02 masneyb Exp $";
 
 static void
 delete_purge_cache (gpointer key, gpointer value, gpointer user_data)
 {
-  gftp_request * request;
-  char *olddir;
-
-  request = user_data;
-  olddir = request->directory;
-  request->directory = key;
-  gftp_delete_cache_entry (request, 0);
-  request->directory = olddir;
+  gftp_delete_cache_entry (NULL, key, 0);
   g_free (key);
 }
 
@@ -38,8 +31,8 @@ delete_purge_cache (gpointer key, gpointer value, gpointer user_data)
 static void *
 do_delete_thread (void *data)
 {
+  char *tempstr, description[BUFSIZ];
   gftp_transfer * transfer;
-  char *pos, *tempstr;
   gftp_file * tempfle;
   GHashTable * rmhash;
   GList * templist;
@@ -61,7 +54,8 @@ do_delete_thread (void *data)
            templist = templist->next);
 
       rmhash = g_hash_table_new (string_hash_function, string_hash_compare);
-      while (1)
+
+      for (; templist != NULL; templist = templist->prev)
         {
           tempfle = templist->data;
           if (tempfle->isdir)
@@ -69,31 +63,29 @@ do_delete_thread (void *data)
           else
             success = gftp_remove_file (transfer->fromreq, tempfle->file);
 
-          if (success == 0 && (pos = strrchr (tempfle->file, '/')))
+          if (success == 0 && transfer->fromreq->use_cache)
             {
-              *pos = '\0';
-              if (g_hash_table_lookup (rmhash, tempfle->file) == NULL)
+              gftp_generate_cache_description (transfer->fromreq, description, 
+                                               sizeof (description), 0);
+              if (g_hash_table_lookup (rmhash, description) == NULL)
                 {
-                  tempstr = g_strconcat (tempfle->file, NULL);
+                  tempstr = g_strdup (description);
                   g_hash_table_insert (rmhash, tempstr, NULL);
                 }
-              *pos = '/';
             }
 
-          if (templist == transfer->files || 
-              !GFTP_IS_CONNECTED (transfer->fromreq))
+          if (!GFTP_IS_CONNECTED (transfer->fromreq))
             break;
-          templist = templist->prev;
         }
 
-      g_hash_table_foreach (rmhash, delete_purge_cache, transfer->fromreq);
+      g_hash_table_foreach (rmhash, delete_purge_cache, NULL);
       g_hash_table_destroy (rmhash);
     }
   else
     {
       gftp_disconnect (transfer->fromreq);
       transfer->fromreq->logging_function (gftp_logging_error,
-                                        transfer->fromreq->user_data,
+                                        transfer->fromreq,
                                         _("Operation canceled\n"));
     }
 
@@ -185,7 +177,7 @@ delete_dialog (gpointer data)
     return;
 
   transfer = g_malloc0 (sizeof (*transfer));
-  transfer->fromreq = copy_request (wdata->request);
+  transfer->fromreq = copy_request (wdata->request, 0);
   transfer->fromwdata = wdata;
   transfer->transfer_direction = GFTP_DIRECTION_DOWNLOAD;
 
