@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  gftp-gtk.h - include file for the gftp gtk+ 1.2 port                     */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -17,7 +17,7 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-/* $Id: gftp-gtk.h,v 1.10 2002/11/23 13:45:05 masneyb Exp $ */
+/* $Id: gftp-gtk.h,v 1.21 2003/08/12 01:05:02 masneyb Exp $ */
 
 #ifndef __GFTP_GTK_H
 #define __GFTP_GTK_H
@@ -65,8 +65,6 @@ typedef struct gftp_window_data_tag
    				   a directory */
             *hoststxt, 		/* Show which directory we're in */
             *listbox; 		/* Our listbox showing the files */
-  unsigned int *sortcol, 	/* Which column we are sorting by */
-               *sortasds; 	/* Sorted ascending or descending */
   unsigned int sorted : 1,	/* Is the output sorted? */
                show_selected : 1, /* Show only selected files */
                *histlen;	/* Pointer to length of history */
@@ -77,7 +75,7 @@ typedef struct gftp_window_data_tag
   GtkItemFactory *ifactory; 	/* This is for the menus that will
                                    come up when you right click */
   pthread_t tid;		/* Thread for the stop button */
-  unsigned long gotbytes;
+  char *prefix_col_str;
 } gftp_window_data;
 
 
@@ -135,6 +133,28 @@ typedef struct gftp_save_dir_struct_tag
 } gftp_save_dir_struct;
 
 
+typedef struct gftp_textcomboedt_widget_data_tag
+{
+  GtkWidget * combo,
+            * text;
+  gftp_config_vars * cv;
+  char * custom_edit_value;
+} gftp_textcomboedt_widget_data;
+
+
+typedef struct gftp_options_dialog_data_tag
+{
+  GtkWidget * dialog,
+            * notebook,
+            * box,
+            * table;
+  int tbl_col_num,
+      tbl_row_num;
+  gftp_option_type_enum last_option;
+  gftp_bookmarks_var * bm;
+} gftp_options_dialog_data;
+
+
 extern gftp_window_data window1, window2, * other_wdata, * current_wdata;
 extern GtkWidget * stop_btn, * hostedit, * useredit, * passedit,
                  * portedit, * logwdw, * dlwdw, * protocol_menu, * optionmenu;
@@ -151,6 +171,9 @@ extern gftp_graphic * gftp_icon;
 extern sigjmp_buf jmp_environment;
 extern volatile int use_jmp_environment;
 extern pthread_t main_thread_id;
+extern GList * viewedit_processes;
+extern volatile sig_atomic_t viewedit_process_done;
+
 
 /* bookmarks.c */
 void run_bookmark 				( gpointer data );
@@ -242,9 +265,9 @@ void savelog 					( gpointer data );
 
 void clear_cache				( gpointer data );
 
-void options_dialog (gpointer data);
-void compare_windows (gpointer data);
-void about_dialog (gpointer data);
+void compare_windows 				( gpointer data );
+
+void about_dialog 				( gpointer data );
 
 /* misc-gtk.c */
 void fix_display				( void );
@@ -252,9 +275,9 @@ void fix_display				( void );
 void remove_files_window			( gftp_window_data * wdata );
 
 void ftp_log					( gftp_logging_level level,
-						  void *ptr,
+						  gftp_request * request,
 						  const char *string,
-						  ... );
+						  ... ) __attribute__((format(printf, 3, 4)));
 
 void refresh 					( gftp_window_data * wdata );
 
@@ -292,10 +315,6 @@ void create_item_factory 			( GtkItemFactory * ifactory,
 						  GtkItemFactoryEntry * entries,
 						  gpointer callback_data );
 
-GList * get_next_selection 			( GList * selection, 
-						  GList ** list, 
-						  int *curnum );
-
 void add_history 				( GtkWidget * widget, 
 						  GList ** history, 
 						  unsigned int *histlen, 
@@ -336,9 +355,20 @@ void display_cached_logs			( void );
 
 RETSIGTYPE signal_handler			(int signo);
 
+char * get_xpm_path 				( char *filename, 
+						  int quit_on_err );
+
 /* mkdir_dialog.c */
 void mkdir_dialog 				( gpointer data );
 
+/* options_dialog.c */
+void options_dialog 				( gpointer data );
+
+void gftp_gtk_setup_bookmark_options 		( GtkWidget * notebook,
+						  gftp_bookmarks_var * bm );
+
+void gftp_gtk_save_bookmark_options 		( gftp_bookmarks_var * bm );
+
 /* rename_dialog.c */
 void rename_dialog				( gpointer data );
 
