@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  menu-items.c - menu callbacks                                            */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include <gftp-gtk.h>
-static const char cvsid[] = "$Id: menu-items.c,v 1.12 2002/11/11 23:16:12 masneyb Exp $";
+static const char cvsid[] = "$Id: menu-items.c,v 1.24 2003/08/12 01:05:02 masneyb Exp $";
 
 void
 change_setting (gftp_window_data * wdata, int menuitem, GtkWidget * checkmenu)
@@ -26,12 +26,10 @@ change_setting (gftp_window_data * wdata, int menuitem, GtkWidget * checkmenu)
   switch (menuitem)
     {
     case 1:
-      if (wdata->request != NULL)
-        gftp_set_data_type (wdata->request, GFTP_TYPE_ASCII);
+      gftp_set_global_option ("ascii_transfers", GINT_TO_POINTER(1));
       break;
     case 2:
-      if (wdata->request != NULL)
-        gftp_set_data_type (wdata->request, GFTP_TYPE_BINARY);
+      gftp_set_global_option ("ascii_transfers", GINT_TO_POINTER(0));
       break;
     case 3:
       current_wdata = &window1;
@@ -109,10 +107,8 @@ do_openurl (gftp_window_data * wdata, gftp_dialog_data * ddata)
       ftp_connect (wdata, wdata->request, 1);
     }
   else
-    {
-      gtk_widget_destroy (ddata->dialog);
-      ftp_log (gftp_logging_misc, NULL, _("Could not parse URL %s\n"), tempstr);    
-    }
+    gtk_widget_destroy (ddata->dialog);
+
   ddata->dialog = NULL; 
 }
 
@@ -132,7 +128,7 @@ disconnect (gpointer data)
   gftp_window_data * wdata;
 
   wdata = data;
-  gftp_delete_cache_entry (wdata->request, 1);
+  gftp_delete_cache_entry (wdata->request, NULL, 1);
   gftp_disconnect (wdata->request);
   remove_files_window (wdata);
   update_window (wdata);
@@ -158,8 +154,7 @@ dochange_filespec (gftp_window_data * wdata, gftp_dialog_data * ddata)
     }
   if (wdata->filespec)
     g_free (wdata->filespec);
-  wdata->filespec = g_malloc (strlen (edttext) + 1);
-  strcpy (wdata->filespec, edttext);
+  wdata->filespec = g_strdup (edttext);
 
   filelist = wdata->files;
   templist = GTK_CLIST (wdata->listbox)->selection;
@@ -211,14 +206,12 @@ destroy_save_directory_listing (GtkWidget * widget, gftp_save_dir_struct * str)
 static void
 dosave_directory_listing (GtkWidget * widget, gftp_save_dir_struct * str)
 {
-  const char *months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
-    "Aug", "Sep", "Oct", "Nov", "Dec" };
   const char *filename;
-  struct tm *lt;
   gftp_file * tempfle;
   GList * templist;
+  char *tempstr;
   FILE * fd;
-  time_t t;
+ 
 
   filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (str->filew));
   if ((fd = fopen (filename, "w")) == NULL)
@@ -229,39 +222,15 @@ dosave_directory_listing (GtkWidget * widget, gftp_save_dir_struct * str)
       return;
     }
 
-  time (&t);
-
   for (templist = str->wdata->files; 
        templist->next != NULL;
        templist = templist->next)
     {
       tempfle = templist->data;
-      fprintf (fd, "%10s %8s %8s ", tempfle->attribs, tempfle->user, 
-               tempfle->group);
-      if (tempfle->attribs && (*tempfle->attribs == 'b' || 
-                               *tempfle->attribs == 'c'))
-        fprintf (fd, "%5d, %4d", (((unsigned int) tempfle->size) >> 16) & 0xFF,
-                                 ((unsigned int) tempfle->size) & 0xFF);
-      else
-        {
-#if defined (_LARGEFILE_SOURCE)
-          fprintf (fd, "%11lld", tempfle->size);
-#else
-          fprintf (fd, "%11ld", tempfle->size);
-#endif
-        }
 
-      lt = localtime (&tempfle->datetime);
-
-      if (tempfle->datetime > t ||
-          t - 3600*24*90 > tempfle->datetime)
-        fprintf (fd, " %s %2d  %4d", months[lt->tm_mon], lt->tm_mday, 
-                 lt->tm_year + 1900);
-      else
-        fprintf (fd, " %s %2d %02d:%02d", months[lt->tm_mon], lt->tm_mday, 
-                 lt->tm_hour, lt->tm_min);
-
-      fprintf (fd, " %s\n", tempfle->file);
+      tempstr = gftp_gen_ls_string (tempfle, NULL, NULL);
+      fprintf (fd, "%s\n", tempstr);
+      g_free (tempstr);
     }
 
   fclose (fd);
@@ -421,8 +390,8 @@ site_dialog (gpointer data)
 static void *
 do_change_dir_thread (void * data)
 {
+  int success, sj, network_timeout;
   gftp_window_data * wdata;
-  int success, sj;
 
   wdata = data;
 
@@ -434,11 +403,14 @@ do_change_dir_thread (void * data)
   else
     sj = 0;
 
+  gftp_lookup_request_option (wdata->request, "network_timeout", 
+                              &network_timeout);
+
   success = 0;
   if (sj == 0) 
     {
-      if (wdata->request->network_timeout > 0)
-        alarm (wdata->request->network_timeout);
+      if (network_timeout > 0)
+        alarm (network_timeout);
       success = gftp_set_directory (wdata->request, wdata->request->directory);
       alarm (0);
     }
@@ -446,7 +418,7 @@ do_change_dir_thread (void * data)
     {
       gftp_disconnect (wdata->request);
       wdata->request->logging_function (gftp_logging_error,
-                                        wdata->request->user_data,
+                                        wdata->request,
                                         _("Operation canceled\n"));
     }
 
@@ -454,7 +426,7 @@ do_change_dir_thread (void * data)
     use_jmp_environment = 0;
 
   wdata->request->stopable = 0;
-  return ((void *) success);
+  return (GINT_TO_POINTER (success));
 }
 
 
@@ -467,13 +439,12 @@ do_change_dir (gftp_window_data * wdata, char *directory)
   if (directory != wdata->request->directory)
     {
       olddir = wdata->request->directory;
-      wdata->request->directory = g_malloc (strlen (directory) + 1);
-      strcpy (wdata->request->directory, directory);
+      wdata->request->directory = g_strdup (directory);
     }
   else
     olddir = NULL;
 
-  ret = (int) generic_thread (do_change_dir_thread, wdata);
+  ret = GPOINTER_TO_INT (generic_thread (do_change_dir_thread, wdata));
 
   if (!GFTP_IS_CONNECTED (wdata->request))
     {
@@ -554,11 +525,14 @@ chdir_dialog (gpointer data)
   templist = get_next_selection (templist, &filelist, &num);
   tempfle = filelist->data;
 
-  newdir = g_strconcat (GFTP_GET_DIRECTORY (wdata->request), "/",
-		        tempfle->file, NULL);
-  remove_double_slashes (newdir);
+  newdir = gftp_build_path (wdata->request->directory, tempfle->file, NULL);
+
   if ((tempstr = expand_path (newdir)) == NULL)
-    return (0);
+    {
+      g_free (newdir);
+      return (0);
+    }
+
   g_free (newdir);
 
   if (check_reconnect (wdata) < 0)
@@ -638,7 +612,9 @@ viewlog (gpointer data)
     {
       if ((len = write (fd, pos, textlen)) == -1)
         { 
-          ftp_log (gftp_logging_misc, NULL, _("Error: Error writing to %s: %s\n"), tempstr, g_strerror (errno));
+          ftp_log (gftp_logging_misc, NULL, 
+                   _("Error: Error writing to %s: %s\n"), 
+                   tempstr, g_strerror (errno));
           break;
         }
       textlen -= len;
@@ -694,7 +670,9 @@ dosavelog (GtkWidget * widget, GtkFileSelection * fs)
       if ((len = write (fileno (fd), pos, textlen)) == -1)
         {
           ok = 0;
-          ftp_log (gftp_logging_misc, NULL, _("Error: Error writing to %s: %s\n"), filename, g_strerror (errno));
+          ftp_log (gftp_logging_misc, NULL, 
+                   _("Error: Error writing to %s: %s\n"), 
+                   filename, g_strerror (errno));
           break;
         }
 
@@ -776,7 +754,7 @@ about_dialog (gpointer data)
     {
       gdk_window_set_icon (dialog->window, NULL, gftp_icon->pixmap,
                            gftp_icon->bitmap);
-      gdk_window_set_icon_name (dialog->window, _("gFTP Icon"));
+      gdk_window_set_icon_name (dialog->window, gftp_version);
     }
 
   notebook = gtk_notebook_new ();
@@ -792,8 +770,7 @@ about_dialog (gpointer data)
   gtk_box_pack_start (GTK_BOX (box), tempwid, FALSE, FALSE, 0);
   gtk_widget_show (tempwid);
 
-  tempstr = g_strdup_printf (_("%s\nCopyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>\nOfficial Homepage: http://www.gftp.org/\nLogo by: Aaron Worley <planet_hoth@yahoo.com>\n"),
-version);
+  tempstr = g_strdup_printf (_("%s\nCopyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>\nOfficial Homepage: http://www.gftp.org/\nLogo by: Aaron Worley <planet_hoth@yahoo.com>\n"), gftp_version);
   str = _("Translated by");
   if (strcmp (str, "Translated by") != 0)
     {
@@ -1018,4 +995,3 @@ compare_windows (gpointer data)
       curlist = curlist->next;
     }
 }
-
