@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  misc-gtk.c - misc stuff for the gtk+ 1.2 port of gftp                    */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,9 +18,8 @@
 /*****************************************************************************/
 
 #include <gftp-gtk.h>
-static const char cvsid[] = "$Id: misc-gtk.c,v 1.19 2002/12/03 02:22:32 masneyb Exp $";
+static const char cvsid[] = "$Id: misc-gtk.c,v 1.32 2003/08/12 01:05:02 masneyb Exp $";
 
-static pthread_mutex_t netfunclock = PTHREAD_MUTEX_INITIALIZER;
 static GtkWidget * statuswid;
 
 
@@ -37,8 +36,10 @@ remove_files_window (gftp_window_data * wdata)
 
 
 void
-ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
+ftp_log (gftp_logging_level level, gftp_request * request, 
+         const char *string, ...)
 {
+  guint max_log_window_size;
   int upd, free_logstr;
   gftp_log * newlog;
   char *logstr;
@@ -49,27 +50,12 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
   gftp_color * color;
   GdkColor fore;
 #else
+  char *utf8_str;
   GtkTextBuffer * textbuf;
   GtkTextIter iter, iter2;
-  gsize bread, bwrite;
   const char *descr;
-  char *tempstr;
 #endif
 
-  if (pthread_self () != main_thread_id)
-    {
-      newlog = g_malloc0 (sizeof (*newlog));
-      newlog->type = level;
-      va_start (argp, string);
-      newlog->msg = g_strdup_vprintf (string, argp);
-      va_end (argp);
-
-      pthread_mutex_lock (&log_mutex);
-      file_transfer_logs = g_list_append (file_transfer_logs, newlog);
-      pthread_mutex_unlock (&log_mutex);
-      return;
-    }
-
   va_start (argp, string);
   if (strcmp (string, "%s") == 0)
     {
@@ -83,40 +69,72 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
     }
   va_end (argp);
 
-  if (logfd != NULL)
+#if GTK_MAJOR_VERSION > 1
+  if (!g_utf8_validate (logstr, -1, NULL))
     {
-      if (fwrite (logstr, strlen (logstr), 1, logfd) != 1)
+      if ((utf8_str = gftp_string_to_utf8 (request, logstr)) != NULL)
         {
-          fclose (logfd);
-          logfd = NULL;
+          if (free_logstr)
+            g_free (logstr);
+          else
+            free_logstr = 1;
+
+          logstr = utf8_str;
+        }
+    }
+#endif
+
+  if (pthread_self () != main_thread_id)
+    {
+      newlog = g_malloc0 (sizeof (*newlog));
+      newlog->type = level;
+      if (free_logstr)
+        newlog->msg = logstr;
+      else
+        newlog->msg = g_strdup (logstr);
+
+      pthread_mutex_lock (&log_mutex);
+      gftp_file_transfer_logs = g_list_append (gftp_file_transfer_logs, newlog);
+      pthread_mutex_unlock (&log_mutex);
+      return;
+    }
+
+  if (gftp_logfd != NULL)
+    {
+      if (fwrite (logstr, strlen (logstr), 1, gftp_logfd) != 1)
+        {
+          fclose (gftp_logfd);
+          gftp_logfd = NULL;
         }
       else
         {
-          fflush (logfd);
-          if (ferror (logfd))
+          fflush (gftp_logfd);
+          if (ferror (gftp_logfd))
             {
-              fclose (logfd);
-              logfd = NULL;
+              fclose (gftp_logfd);
+              gftp_logfd = NULL;
             }
         }
     }
 
   upd = logwdw_vadj->upper - logwdw_vadj->page_size == logwdw_vadj->value;
 
+  gftp_lookup_global_option ("max_log_window_size", &max_log_window_size);
+
 #if GTK_MAJOR_VERSION == 1
   switch (level)
     {
       case gftp_logging_send:
-        color = &send_color;
+        gftp_lookup_global_option ("send_color", &color);
         break;
       case gftp_logging_recv:
-        color = &recv_color;
+        gftp_lookup_global_option ("recv_color", &color);
         break;
       case gftp_logging_error:
-        color = &error_color;
+        gftp_lookup_global_option ("error_color", &color);
         break;
       default:
-        color = &misc_color;
+        gftp_lookup_global_option ("misc_color", &color);
         break;
     }
 
@@ -159,20 +177,6 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
         break;
     }
 
-  /* If the current log message is not in UTF8 format, convert it to UTF8 
-     format based on the current locale */
-  if (!g_utf8_validate (logstr, -1, NULL))
-    {
-      tempstr = g_locale_to_utf8 (logstr, -1, &bread, &bwrite, NULL);
-      if (tempstr != NULL)
-        {
-          if (free_logstr)
-            g_free (logstr);
-          logstr = tempstr;
-          free_logstr = 1;
-        }
-    }
-
   textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (logwdw));
   len = gtk_text_buffer_get_char_count (textbuf);
   gtk_text_buffer_get_iter_at_offset (textbuf, &iter, len);
@@ -211,7 +215,7 @@ refresh (gftp_window_data * wdata)
     return;
   gtk_clist_freeze (GTK_CLIST (wdata->listbox));
   remove_files_window (wdata);
-  gftp_delete_cache_entry (wdata->request, 0);
+  gftp_delete_cache_entry (wdata->request, NULL, 0);
   ftp_list_files (wdata, 0);
   gtk_clist_thaw (GTK_CLIST (wdata->listbox));
 }
@@ -226,21 +230,21 @@ update_window_info (void)
 
   if (current_wdata->request != NULL)
     {
-      if ((tempstr = GFTP_GET_HOSTNAME (current_wdata->request)) == NULL)
+      if ((tempstr = current_wdata->request->hostname) == NULL)
         tempstr = empty;
       gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry), tempstr);
 
-      if ((tempstr = GFTP_GET_USERNAME (current_wdata->request)) == NULL)
+      if ((tempstr = current_wdata->request->username) == NULL)
         tempstr = empty;
       gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (useredit)->entry), tempstr);
 
-      if ((tempstr = GFTP_GET_PASSWORD (current_wdata->request)) == NULL)
+      if ((tempstr = current_wdata->request->password) == NULL)
         tempstr = empty;
       gtk_entry_set_text (GTK_ENTRY (passedit), tempstr);
 
-      if (GFTP_GET_PORT (current_wdata->request) != 0)
+      if (current_wdata->request->port != 0)
         {
-          tempstr = g_strdup_printf ("%d", GFTP_GET_PORT (current_wdata->request));
+          tempstr = g_strdup_printf ("%d", current_wdata->request->port);
           gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (portedit)->entry), tempstr);
           g_free (tempstr);
         }
@@ -298,17 +302,17 @@ update_window (gftp_window_data * wdata)
     {
       fspec = wdata->show_selected ? "Selected" : strcmp (wdata->filespec, "*") == 0 ?  _("All Files") : wdata->filespec;
 
-      if ((temp1str = GFTP_GET_HOSTNAME (wdata->request)) == NULL ||
+      if ((temp1str = wdata->request->hostname) == NULL ||
           wdata->request->protonum == GFTP_LOCAL_NUM)
 	temp1str = "";
       tempstr = g_strconcat (temp1str, *temp1str == '\0' ? "[" : " [",
-		     GFTP_GET_PROTOCOL_NAME (wdata->request),
+		     gftp_protocols[wdata->request->protonum].name,
 		     wdata->request->cached ? _("] (Cached) [") : "] [",
                      fspec, "]", current_wdata == wdata ? "*" : "", NULL);
       gtk_label_set (GTK_LABEL (wdata->hoststxt), tempstr);
       g_free (tempstr);
 
-      if ((dir = GFTP_GET_DIRECTORY (wdata->request)) == NULL)
+      if ((dir = wdata->request->directory) == NULL)
         temp1str = "";
       else
         temp1str = dir;
@@ -408,8 +412,7 @@ open_xpm (GtkWidget * widget, char *filename)
       return (NULL);
     }
 
-  graphic->filename = g_malloc (strlen (filename) + 1);
-  strcpy (graphic->filename, filename);
+  graphic->filename = g_strdup (filename);
   g_hash_table_insert (graphic_hash_table, graphic->filename, graphic);
 
   return (graphic);
@@ -523,8 +526,7 @@ check_status (char *name, gftp_window_data *wdata, int check_other_stop,
 
 
 static gchar *
-gftp_item_factory_translate (const char *path,
-			     gpointer    func_data)
+gftp_item_factory_translate (const char *path, gpointer func_data)
 {
   const gchar *strip_prefix = func_data;
   const char *result;
@@ -534,7 +536,7 @@ gftp_item_factory_translate (const char *path,
       char *tmp_path = g_strconcat (strip_prefix, path, NULL);
       result = gettext (tmp_path);
       if (result == tmp_path)
-	result = path;
+        result = path;
       g_free (tmp_path);
     }
   else
@@ -549,7 +551,7 @@ item_factory_new (GtkType container_type, const char *path,
 		  GtkAccelGroup *accel_group, const char *strip_prefix)
 {
   GtkItemFactory *result = gtk_item_factory_new (container_type, path, accel_group);
-  gchar *strip_prefix_dup = g_strdup (g_strdup (strip_prefix));
+  gchar *strip_prefix_dup = g_strdup (strip_prefix);
   
   gtk_item_factory_set_translate_func (result, gftp_item_factory_translate,
 				       strip_prefix_dup, NULL);
@@ -583,52 +585,6 @@ create_item_factory (GtkItemFactory * ifactory, guint n_entries,
     }
 }
 
-GList *
-get_next_selection (GList * selection, GList ** list, int *curnum)
-{
-  gftp_file * tempfle;
-  int i, newpos;
-
-  newpos = (int) selection->data;
-  i = *curnum - newpos;
-
-  if (i < 0)
-    {
-      while (i != 0)
-        {
-          tempfle = (*list)->data;
-          if (tempfle->shown)
-            {
-	      ++*curnum;
-	      i++;
-            }
-	  *list = (*list)->next;
-        }     
-    }
-  else if (i > 0)
-    {
-      while (i != 0)
-        {
-          tempfle = (*list)->data;
-          if (tempfle->shown)
-            {
-	      --*curnum;
-	      i--;
-            }
-	  *list = (*list)->prev;
-        }
-    }
-
-  tempfle = (*list)->data;
-  while ((*list)->next && !tempfle->shown)
-    {
-      *list = (*list)->next;
-      tempfle = (*list)->data;
-    }
-  return (selection->next);
-}
-
-
 void
 add_history (GtkWidget * widget, GList ** history, unsigned int *histlen, 
              const char *str)
@@ -664,8 +620,7 @@ add_history (GtkWidget * widget, GList ** history, unsigned int *histlen,
 	    }
 	  g_list_free (node);
 	}
-      tempstr = g_malloc (strlen (str) + 1);
-      strcpy (tempstr, str);
+      tempstr = g_strdup (str);
       *history = g_list_prepend (*history, tempstr);
       ++*histlen;
     }
@@ -687,7 +642,7 @@ add_history (GtkWidget * widget, GList ** history, unsigned int *histlen,
 int
 check_reconnect (gftp_window_data *wdata)
 {
-  return (wdata->request->cached && wdata->request->sockfd < 0 && 
+  return (wdata->request->cached && wdata->request->datafd < 0 && 
           !wdata->request->always_connected &&
 	  !ftp_connect (wdata, wdata->request, 0) ? -1 : 0);
 }
@@ -697,14 +652,18 @@ void
 add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
 {
   char *add_data[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL }, *pos;
+  gftp_config_list_vars * tmplistvar;
+  int clist_num, show_hidden_files;
   gftp_file_extensions * tempext;
   char *tempstr, *str;
   GdkBitmap * bitmap;
   GList * templist;
   GdkPixmap * pix;
-  int clist_num;
   size_t stlen;
 
+  gftp_lookup_request_option (wdata->request, "show_hidden_files", 
+                              &show_hidden_files);
+
   if (wdata->show_selected)
     {
       fle->shown = fle->was_sel;
@@ -745,16 +704,14 @@ add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
   else
     {
       stlen = strlen (fle->file);
-      templist = registered_exts;
+      gftp_lookup_global_option ("ext", &tmplistvar);
+      templist = tmplistvar->list;
       while (templist != NULL)
         {
           tempext = templist->data;
           if (stlen >= tempext->stlen &&
               strcmp (&fle->file[stlen - tempext->stlen], tempext->ext) == 0)
             {
-              if (toupper (*tempext->ascii_binary) == 'A')
-                fle->ascii = 1;
-
               gftp_get_pixmap (wdata->listbox, tempext->filename, &pix, 
                                &bitmap);
               break;
@@ -768,7 +725,10 @@ add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
    
   gtk_clist_set_pixmap (GTK_CLIST (wdata->listbox), clist_num, 0, pix, bitmap);
 
-  if (fle->file)
+  if (fle->utf8_file)
+    gtk_clist_set_text (GTK_CLIST (wdata->listbox), clist_num, 1, 
+                        fle->utf8_file);
+  else if (fle->file)
     gtk_clist_set_text (GTK_CLIST (wdata->listbox), clist_num, 1, fle->file);
 
   if (fle->attribs && (*fle->attribs == 'b' || *fle->attribs == 'c'))
@@ -801,7 +761,11 @@ static void
 ok_dialog_response (GtkWidget * widget, gftp_dialog_data * ddata)
 {
   if (ddata->edit == NULL)
-    gtk_widget_destroy (ddata->dialog);
+    {
+      gtk_widget_destroy (ddata->dialog);
+      ddata->dialog = NULL;
+      ddata->checkbox = NULL;
+    }
  
   if (ddata->yesfunc != NULL)
     ddata->yesfunc (ddata->yespointer, ddata);
@@ -817,7 +781,11 @@ static void
 cancel_dialog_response (GtkWidget * widget, gftp_dialog_data * ddata)
 {
   if (ddata->edit == NULL)
-    gtk_widget_destroy (ddata->dialog);
+    {
+      gtk_widget_destroy (ddata->dialog);
+      ddata->dialog = NULL;
+      ddata->checkbox = NULL;
+    }
  
   if (ddata->nofunc != NULL)
     ddata->nofunc (ddata->nopointer, ddata);
@@ -832,7 +800,11 @@ static void
 dialog_response (GtkWidget * widget, gint response, gftp_dialog_data * ddata)
 {
   if (ddata->edit == NULL)
-    gtk_widget_destroy (ddata->dialog);
+    {
+      gtk_widget_destroy (ddata->dialog);
+      ddata->dialog = NULL;
+      ddata->checkbox = NULL;
+    }
 
   switch (response)
     {
@@ -944,7 +916,7 @@ MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
     {
       gdk_window_set_icon (dialog->window, NULL, gftp_icon->pixmap,
                            gftp_icon->bitmap);
-      gdk_window_set_icon_name (dialog->window, _("gFTP Icon"));
+      gdk_window_set_icon_name (dialog->window, gftp_version);
     }
 
   ddata->dialog = dialog;
@@ -1068,7 +1040,7 @@ MakeYesNoDialog (char *diagtxt, char *infotxt,
     {
       gdk_window_set_icon (dialog->window, NULL, gftp_icon->pixmap,
                            gftp_icon->bitmap);
-      gdk_window_set_icon_name (dialog->window, _("gFTP Icon"));
+      gdk_window_set_icon_name (dialog->window, gftp_version);
     }
 
   ddata->dialog = dialog;
@@ -1242,18 +1214,18 @@ display_cached_logs (void)
   GList * templist; 
 
   pthread_mutex_lock (&log_mutex);
-  templist = file_transfer_logs;
+  templist = gftp_file_transfer_logs;
   while (templist != NULL)
     { 
       templog = (gftp_log *) templist->data;
-      ftp_log (templog->type, (void *) 0x2, "%s", templog->msg);
+      ftp_log (templog->type, NULL, "%s", templog->msg);
       g_free (templog->msg);
       g_free (templog); 
       templist->data = NULL;
       templist = templist->next;
     }
-  g_list_free (file_transfer_logs);
-  file_transfer_logs = NULL;
+  g_list_free (gftp_file_transfer_logs);
+  gftp_file_transfer_logs = NULL;
   pthread_mutex_unlock (&log_mutex);
 }
 
@@ -1270,48 +1242,36 @@ signal_handler (int signo)
 }
 
 
-#if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
-
-struct hostent *
-r_gethostbyname (const char *name, struct hostent *result_buf, int *h_errnop)
+char *
+get_xpm_path (char *filename, int quit_on_err)
 {
-  struct hostent *hent;
+  char *tempstr, *exfile;
 
-  pthread_mutex_lock (&netfunclock);
-  if ((hent = gethostbyname (name)) == NULL)
+  tempstr = g_strconcat (BASE_CONF_DIR, "/", filename, NULL);
+  exfile = expand_path (tempstr);
+  g_free (tempstr);
+  if (access (exfile, F_OK) != 0)
     {
-      if (h_errnop)
-        *h_errnop = h_errno;
+      g_free (exfile);
+      tempstr = g_strconcat (SHARE_DIR, "/", filename, NULL);
+      exfile = expand_path (tempstr);
+      g_free (tempstr);
+      if (access (exfile, F_OK) != 0)
+	{
+	  g_free (exfile);
+	  exfile = g_strconcat ("/usr/share/icons/", filename, NULL);
+	  if (access (exfile, F_OK) != 0)
+	    {
+	      g_free (exfile);
+	      if (!quit_on_err)
+		return (NULL);
+
+	      printf (_("gFTP Error: Cannot find file %s in %s or %s\n"),
+		      filename, SHARE_DIR, BASE_CONF_DIR);
+	      exit (1);
+	    }
+	}
     }
-  else
-    {
-      *result_buf = *hent;
-      hent = result_buf;
-    }
-  pthread_mutex_unlock (&netfunclock);
-  return (hent);
-}
-
-#endif /* !HAVE_GETADDRINFO */
-
-struct servent *
-r_getservbyname (const char *name, const char *proto,
-                 struct servent *result_buf, int *h_errnop)
-{
-  struct servent *sent;
-
-  pthread_mutex_lock (&netfunclock);
-  if ((sent = getservbyname (name, proto)) == NULL)
-    {
-      if (h_errnop)
-        *h_errnop = h_errno;
-    }
-  else
-    {
-      *result_buf = *sent;
-      sent = result_buf;
-    }
-  pthread_mutex_unlock (&netfunclock);
-  return (sent);
+  return (exfile);
 }
 
