@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  mkdir_dialog.c - make directory dialog box and ftp routines              */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: mkdir_dialog.c,v 1.6 2002/11/11 23:16:12 masneyb Exp $";
+static const char cvsid[] = "$Id: mkdir_dialog.c,v 1.11 2003/08/12 01:05:02 masneyb Exp $";
 
 static const char *edttext;
 
@@ -26,11 +26,14 @@ static const char *edttext;
 static void *
 do_make_dir_thread (void * data)
 {
+  int success, sj, network_timeout;
   gftp_window_data * wdata;
-  int success, sj;
 
   wdata = data;
 
+  gftp_lookup_request_option (wdata->request, "network_timeout", 
+                              &network_timeout);
+
   if (wdata->request->use_threads)
     {
       sj = sigsetjmp (jmp_environment, 1);
@@ -42,8 +45,8 @@ do_make_dir_thread (void * data)
   success = 0;
   if (sj == 0)
     {
-      if (wdata->request->network_timeout > 0)
-        alarm (wdata->request->network_timeout);
+      if (network_timeout > 0)
+        alarm (network_timeout);
       success = gftp_make_directory (wdata->request, edttext) == 0;
       alarm (0);
     }
@@ -51,7 +54,7 @@ do_make_dir_thread (void * data)
     {
       gftp_disconnect (wdata->request);
       wdata->request->logging_function (gftp_logging_error,
-                                        wdata->request->user_data,
+                                        wdata->request,
                                         _("Operation canceled\n"));
     }
 
@@ -59,13 +62,15 @@ do_make_dir_thread (void * data)
     use_jmp_environment = 0;
 
   wdata->request->stopable = 0;
-  return ((void *) success);
+  return (GINT_TO_POINTER (success));
 }
 
 
 static void
 domkdir (gftp_window_data * wdata, gftp_dialog_data * ddata)
 {
+  int ret;
+
   edttext = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
   if (*edttext == '\0')
     {
@@ -77,9 +82,10 @@ domkdir (gftp_window_data * wdata, gftp_dialog_data * ddata)
   if (check_reconnect (wdata) < 0)
     return;
 
-  if ((int) generic_thread (do_make_dir_thread, wdata))
+  ret = GPOINTER_TO_INT (generic_thread (do_make_dir_thread, wdata));
+  if (ret)
     {
-      gftp_delete_cache_entry (wdata->request, 0);
+      gftp_delete_cache_entry (wdata->request, NULL, 0);
       refresh (wdata);
     }
 }
