@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  rename_dialog.c - rename dialog box and ftp routines                     */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: rename_dialog.c,v 1.5 2002/11/11 23:16:12 masneyb Exp $";
+static const char cvsid[] = "$Id: rename_dialog.c,v 1.9 2003/08/12 01:05:02 masneyb Exp $";
 
 static const char *edttext;
 static gftp_file * curfle;
@@ -27,11 +27,14 @@ static gftp_file * curfle;
 static void *
 do_rename_thread (void * data)
 {
+  int success, sj, network_timeout;
   gftp_window_data * wdata;
-  int success, sj;
 
   wdata = data;
 
+  gftp_lookup_request_option (wdata->request, "network_timeout", 
+                              &network_timeout);
+
   if (wdata->request->use_threads)
     { 
       sj = sigsetjmp (jmp_environment, 1);
@@ -43,8 +46,8 @@ do_rename_thread (void * data)
   success = 0;
   if (sj == 0)
     {
-      if (wdata->request->network_timeout > 0)
-        alarm (wdata->request->network_timeout);
+      if (network_timeout > 0)
+        alarm (network_timeout);
       success = gftp_rename_file (wdata->request, curfle->file, edttext) == 0;
       alarm (0);
     }
@@ -52,7 +55,7 @@ do_rename_thread (void * data)
     {
       gftp_disconnect (wdata->request);
       wdata->request->logging_function (gftp_logging_error,
-                                        wdata->request->user_data,
+                                        wdata->request,
                                         _("Operation canceled\n"));
     }
 
@@ -60,13 +63,15 @@ do_rename_thread (void * data)
     use_jmp_environment = 0;
 
   wdata->request->stopable = 0;
-  return ((void *) success);
+  return (GINT_TO_POINTER (success));
 }
 
 
 static void
 dorenCB (gftp_window_data * wdata, gftp_dialog_data * ddata)
 {
+  int ret;
+
   edttext = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
   if (*edttext == '\0')
     {
@@ -78,7 +83,8 @@ dorenCB (gftp_window_data * wdata, gftp_dialog_data * ddata)
   if (check_reconnect (wdata) < 0) 
     return;
 
-  if ((int) generic_thread (do_rename_thread, wdata))
+  ret = GPOINTER_TO_INT (generic_thread (do_rename_thread, wdata));
+  if (ret)
     refresh ((gpointer) wdata);
 }
 
