@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  gftp-text.c - text port of gftp                                          */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,13 +18,12 @@
 /*****************************************************************************/
 
 #include "gftp-text.h"
-static const char cvsid[] = "$Id: gftp-text.c,v 1.7 2002/11/27 14:29:56 masneyb Exp $";
+static const char cvsid[] = "$Id: gftp-text.c,v 1.26 2003/07/24 01:48:33 masneyb Exp $";
 
 static gftp_request * gftp_text_locreq = NULL;
 static gftp_request * gftp_text_remreq = NULL;
 static volatile int cancel = 0;
-static int configuration_changed = 0,
-           number_commands = 30;
+static int number_commands = 30;
 
 struct _gftp_text_methods gftp_text_methods[] = {
         {N_("about"), 	2, gftp_text_about,	NULL,
@@ -94,11 +93,11 @@ struct _gftp_text_methods gftp_text_methods[] = {
 int
 main (int argc, char **argv)
 {
+  char *pos, *stpos, *startup_directory;
   gftp_request * request;
   size_t len, cmdlen;
-  char *pos, *stpos;
   int i;
-#ifdef HAVE_LIBREADLINE
+#if HAVE_LIBREADLINE
   char *tempstr, prompt[20];
 #else
   char tempstr[512];
@@ -107,32 +106,44 @@ main (int argc, char **argv)
 #ifdef HAVE_GETTEXT
   setlocale (LC_ALL, "");
   bindtextdomain ("gftp", LOCALE_DIR);
-  textdomain ("gftp");
 #endif
 
   signal (SIGCHLD, sig_child);
   signal (SIGPIPE, SIG_IGN); 
 
-  gftp_read_config_file (argv, 0);
-
-  /* SSH doesn't support reading the password with askpass via the command 
-     line */
-  ssh_use_askpass = sshv2_use_sftp_subsys = 0;
+  gftp_read_config_file (SHARE_DIR);
 
   if (gftp_parse_command_line (&argc, &argv) != 0)
     exit (0);
 
+  /* SSH doesn't support reading the password with askpass via the command 
+     line */
+
   gftp_text_remreq = gftp_request_new ();
+  gftp_set_request_option (gftp_text_remreq, "ssh_use_askpass", 
+                           GINT_TO_POINTER(0));
+  gftp_set_request_option (gftp_text_remreq, "sshv2_use_sftp_subsys", 
+                           GINT_TO_POINTER(0));
   gftp_text_remreq->logging_function = gftp_text_log;
 
   gftp_text_locreq = gftp_request_new ();
-  gftp_text_locreq->logging_function = gftp_text_log;
-  gftp_protocols[GFTP_LOCAL_NUM].init (gftp_text_locreq);
-  if (startup_directory != NULL && *startup_directory != '\0')
-    gftp_set_directory (gftp_text_locreq, startup_directory);
-  gftp_connect (gftp_text_locreq);
+  gftp_set_request_option (gftp_text_locreq, "ssh_use_askpass", 
+                           GINT_TO_POINTER(0));
+  gftp_set_request_option (gftp_text_locreq, "sshv2_use_sftp_subsys", 
+                           GINT_TO_POINTER(0));
 
-  gftp_text_log (gftp_logging_misc, NULL, "%s, Copyright (C) 1998-2002 Brian Masney <", version);
+  gftp_text_locreq->logging_function = gftp_text_log;
+  if (gftp_protocols[GFTP_LOCAL_NUM].init (gftp_text_locreq) == 0)
+    {
+      gftp_lookup_request_option (gftp_text_locreq, "startup_directory", 
+                                  &startup_directory);
+      if (*startup_directory != '\0')
+        gftp_set_directory (gftp_text_locreq, startup_directory);
+
+      gftp_connect (gftp_text_locreq);
+    }
+
+  gftp_text_log (gftp_logging_misc, NULL, "%s, Copyright (C) 1998-2003 Brian Masney <", gftp_version);
   gftp_text_log (gftp_logging_recv, NULL, "masneyb@gftp.org");
   gftp_text_log (gftp_logging_misc, NULL, _(">.\nIf you have any questions, comments, or suggestions about this program, please feel free to email them to me. You can always find out the latest news about gFTP from my website at http://www.gftp.org/\n"));
   gftp_text_log (gftp_logging_misc, NULL, "\n");
@@ -154,7 +165,7 @@ main (int argc, char **argv)
   else if (argc == 2)
     gftp_text_open (gftp_text_remreq, argv[1], NULL);
 
-#ifdef HAVE_LIBREADLINE
+#if HAVE_LIBREADLINE
   g_snprintf (prompt, sizeof (prompt), "%sftp%s> ", COLOR_BLUE, COLOR_DEFAULT);
   while ((tempstr = readline (prompt)))
 #else
@@ -177,13 +188,13 @@ main (int argc, char **argv)
 
       if (*stpos == '\0')
         {
-#ifndef HAVE_LIBREADLINE
+#if !HAVE_LIBREADLINE
           printf ("%sftp%s> ", COLOR_BLUE, COLOR_DEFAULT);
 #endif
           continue;
         }
 
-#ifdef HAVE_LIBREADLINE
+#if HAVE_LIBREADLINE
       add_history (tempstr);
 #endif
 
@@ -219,15 +230,13 @@ main (int argc, char **argv)
         gftp_text_log (gftp_logging_error, NULL, 
                        _("Error: Command not recognized\n"));
 
-#ifdef HAVE_LIBREADLINE
+#if HAVE_LIBREADLINE
      free (tempstr);
 #else
      printf ("%sftp%s> ", COLOR_BLUE, COLOR_DEFAULT);
 #endif
     }
  
-  if (logfd != NULL)
-    fclose (logfd);
   gftp_text_quit (NULL, NULL, NULL);
 
   return (0);
@@ -235,9 +244,10 @@ main (int argc, char **argv)
 
 
 void
-gftp_text_log (gftp_logging_level level, void *ptr, const char *string, ...)
+gftp_text_log (gftp_logging_level level, gftp_request * request, 
+               const char *string, ...)
 {
-  char tempstr[512], *stpos, *endpos;
+  char tempstr[512], *stpos, *endpos, *utf8_str = NULL, *outstr;
   va_list argp;
   int sw;
 
@@ -263,21 +273,31 @@ gftp_text_log (gftp_logging_level level, void *ptr, const char *string, ...)
   g_vsnprintf (tempstr, sizeof (tempstr), string, argp);
   va_end (argp);
 
-  if (logfd != NULL)
+#if GLIB_MAJOR_VERSION > 1
+  if (!g_utf8_validate (tempstr, -1, NULL))
+    utf8_str = gftp_string_to_utf8 (request, tempstr);
+#endif
+
+  if (utf8_str != NULL)
+    outstr = utf8_str;
+  else
+    outstr = tempstr;
+
+  if (gftp_logfd != NULL)
     {
-      fwrite (tempstr, 1, strlen (tempstr), logfd);
-      if (ferror (logfd))
+      fwrite (outstr, 1, strlen (outstr), gftp_logfd);
+      if (ferror (gftp_logfd))
         {
-          fclose (logfd);
-          logfd = NULL;
+          fclose (gftp_logfd);
+          gftp_logfd = NULL;
         }
       else
-        fflush (logfd);
+        fflush (gftp_logfd);
     }
 
   sw = gftp_text_get_win_size ();
-  stpos = tempstr;
-  endpos = tempstr + 1;
+  stpos = outstr;
+  endpos = outstr + 1;
   do
     {
       if (strlen (stpos) <= sw)
@@ -296,6 +316,9 @@ gftp_text_log (gftp_logging_level level, void *ptr, const char *string, ...)
   while (stpos != endpos);
   
   printf ("%s", COLOR_DEFAULT);
+
+  if (utf8_str != NULL)
+    g_free (utf8_str);
 }
 
 
@@ -311,41 +334,35 @@ gftp_text_open (gftp_request * request, char *command, gpointer *data)
 
   if (*command == '\0')
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
           _("usage: open [[ftp://][user:pass@]ftp-site[:port][/directory]]\n"));
       return (1);
     }
   
   if (gftp_parse_url (request, command) < 0)
-    {
-      gftp_text_log (gftp_logging_error, NULL, 
-                     _("Could not parse URL %s\n"), command);
-      return (1);
-    }
+    return (1);
 
-  if (GFTP_GET_USERNAME (request) == NULL)
+  if (request->need_userpass)
     {
-      if ((pos = gftp_text_ask_question ("Username [anonymous]", 1, tempstr, 
-                                         sizeof (tempstr))) != NULL)
+      if (request->username == NULL || *request->username == '\0')
         {
-          gftp_set_username (request, pos);
-          if (request->password)
+          if ((pos = gftp_text_ask_question ("Username [anonymous]", 1, tempstr, 
+                                             sizeof (tempstr))) != NULL)
             {
-              g_free (request->password);
-              request->password = NULL;
+              gftp_set_username (request, pos);
+              gftp_set_password (request, NULL);
             }
         }
-      else
-        gftp_set_username (request, "anonymous");
-    }
 
-  if (strcmp (GFTP_GET_USERNAME (request), "anonymous") != 0 && 
-      (request->password == NULL || *request->password == '\0'))
-    {
-      if ((pos = gftp_text_ask_question ("Password", 0, tempstr, 
-                                         sizeof (tempstr))) == NULL)
-        return (1);
-      gftp_set_password (request, pos);
+      if (request->username != NULL &&
+          strcmp (request->username, "anonymous") != 0 && 
+          (request->password == NULL || *request->password == '\0'))
+        {
+          if ((pos = gftp_text_ask_question ("Password", 0, tempstr, 
+                                             sizeof (tempstr))) == NULL)
+            return (1);
+          gftp_set_password (request, pos);
+        }
     }
 
   gftp_connect (request);
@@ -366,12 +383,13 @@ gftp_text_about (gftp_request * request, char *command, gpointer *data)
 {
   char *str;
 
-  gftp_text_log (gftp_logging_misc, NULL,
-      "%s. Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>\n", version);
+  gftp_text_log (gftp_logging_misc, request,
+      "%s. Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>\n", 
+      gftp_version);
 
   str = _("Translated by");
   if (strcmp (str, "Translated by") != 0)
-    gftp_text_log (gftp_logging_misc, NULL, "%s\n", str);
+    gftp_text_log (gftp_logging_misc, request, "%s\n", str);
   return (1);
 }
 
@@ -379,9 +397,10 @@ gftp_text_about (gftp_request * request, char *command, gpointer *data)
 int
 gftp_text_quit (gftp_request * request, char *command, gpointer *data)
 {
-  gftp_clear_cache_files ();
-  if (configuration_changed)
-    gftp_write_config_file ();
+  gftp_request_destroy (gftp_text_locreq, 1);
+  gftp_request_destroy (gftp_text_remreq, 1);
+  gftp_shutdown();
+
   return (0);
 }
 
@@ -391,11 +410,11 @@ gftp_text_pwd (gftp_request * request, char *command, gpointer *data)
 {
   if (!GFTP_IS_CONNECTED (request))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
-  gftp_text_log (gftp_logging_misc, NULL, "%s\n", request->directory);
+  gftp_text_log (gftp_logging_misc, request, "%s\n", request->directory);
   return (1);
 }
 
@@ -403,24 +422,37 @@ gftp_text_pwd (gftp_request * request, char *command, gpointer *data)
 int
 gftp_text_cd (gftp_request * request, char *command, gpointer *data)
 {
-  char *newdir = NULL;
+  char *tempstr, *newdir = NULL;
 
   if (!GFTP_IS_CONNECTED (request))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
   else if (*command == '\0')
     {
-      gftp_text_log (gftp_logging_error, NULL, _("usage: chdir <directory>\n"));
+      gftp_text_log (gftp_logging_error, request, 
+                     _("usage: chdir <directory>\n"));
       return (1);
     }
-  else if (request->protonum == GFTP_LOCAL_NUM &&
-           (newdir = expand_path (command)) == NULL)
+  else if (request->protonum == GFTP_LOCAL_NUM)
     {
-      gftp_text_log (gftp_logging_error, NULL, _("usage: chdir <directory>\n"));
-      return (1);
+      if (*command != '/' && request->directory != NULL)
+        {
+          tempstr = g_strconcat (request->directory, "/", command, NULL);
+          newdir = expand_path (tempstr);
+          g_free (tempstr);
+        }
+      else
+        newdir = expand_path (command);
+
+      if (newdir == NULL)
+        {
+          gftp_text_log (gftp_logging_error, request, 
+                         _("usage: chdir <directory>\n"));
+          return (1);
+        }
     }
 
   gftp_set_directory (request, newdir != NULL ? newdir : command);
@@ -437,14 +469,14 @@ gftp_text_mkdir (gftp_request * request, char *command, gpointer *data)
 {
   if (!GFTP_IS_CONNECTED (request))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
 
   if (*command == '\0')
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("usage: mkdir <new directory>\n"));
     }
   else
@@ -460,14 +492,15 @@ gftp_text_rmdir (gftp_request * request, char *command, gpointer *data)
 {
   if (!GFTP_IS_CONNECTED (request))
     {
-      gftp_text_log (gftp_logging_error, NULL, 
+      gftp_text_log (gftp_logging_error, request, 
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
 
   if (*command == '\0')
     {
-      gftp_text_log (gftp_logging_error, NULL, _("usage: rmdir <directory>\n"));
+      gftp_text_log (gftp_logging_error, request, 
+                     _("usage: rmdir <directory>\n"));
     }
   else
     {
@@ -482,18 +515,20 @@ gftp_text_delete (gftp_request * request, char *command, gpointer *data)
 {
   if (!GFTP_IS_CONNECTED (request))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
 
   if (*command == '\0')
     {
-      gftp_text_log (gftp_logging_error, NULL,_("usage: delete <file>\n"));
+      gftp_text_log (gftp_logging_error, request,
+                     _("usage: delete <file>\n"));
     }
   else
     {
-      gftp_remove_file (request, command);
+      if (gftp_remove_file (request, command) == 0)
+        gftp_delete_cache_entry (request, NULL, 0);
     }
   return (1);
 }
@@ -506,7 +541,7 @@ gftp_text_rename (gftp_request * request, char *command, gpointer *data)
 
   if (!GFTP_IS_CONNECTED (request))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
@@ -516,7 +551,7 @@ gftp_text_rename (gftp_request * request, char *command, gpointer *data)
 
   if (*command == '\0' || pos == NULL || *pos == '\0')
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("usage: rename <old name> <new name>\n"));
     }
   else
@@ -534,7 +569,7 @@ gftp_text_chmod (gftp_request * request, char *command, gpointer *data)
 
   if (!GFTP_IS_CONNECTED (request))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
@@ -544,7 +579,7 @@ gftp_text_chmod (gftp_request * request, char *command, gpointer *data)
 
   if (*command == '\0' || pos == NULL || *pos == '\0')
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("usage: chmod <mode> <file>\n"));
     }
   else
@@ -559,7 +594,7 @@ int
 gftp_text_ls (gftp_request * request, char *command, gpointer *data)
 {
   GList * files, * templist, * delitem;
-  char *color, buf[20], *filespec;
+  char *color, *filespec, *tempstr;
   int sortcol, sortasds;
   gftp_file * fle;
   time_t curtime;
@@ -567,7 +602,7 @@ gftp_text_ls (gftp_request * request, char *command, gpointer *data)
   time (&curtime);
   if (!GFTP_IS_CONNECTED (request))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
@@ -591,17 +626,20 @@ gftp_text_ls (gftp_request * request, char *command, gpointer *data)
   g_free (fle);
 
   if (files == NULL)
-    return (1);
+    {
+      gftp_end_transfer (request);
+      return (1);
+    }
 
   if (request == gftp_text_locreq)
     {
-      sortcol = local_sortcol;
-      sortasds = local_sortasds;
+      gftp_lookup_request_option (request, "local_sortcol", &sortcol);
+      gftp_lookup_request_option (request, "local_sortasds", &sortasds);
     }
   else
     {
-      sortcol = remote_sortcol;
-      sortasds = remote_sortasds;
+      gftp_lookup_request_option (request, "remote_sortcol", &sortcol);
+      gftp_lookup_request_option (request, "remote_sortasds", &sortasds);
     }
 
   files = gftp_sort_filelist (files, sortcol, sortasds);
@@ -627,21 +665,13 @@ gftp_text_ls (gftp_request * request, char *command, gpointer *data)
       else
         color = COLOR_DEFAULT;
 
-      if (curtime > fle->datetime + 6 * 30 * 24 * 60 * 60 ||
-          curtime < fle->datetime - 60 * 60)
-        strftime (buf, sizeof (buf), "%b %d  %Y", localtime (&fle->datetime));
-      else
-        strftime (buf, sizeof (buf), "%b %d %H:%M", localtime (&fle->datetime));
-      
-#if defined (_LARGEFILE_SOURCE)
-      printf ("%s %8s %8s %10lld %s %s%s%s\n", 
-#else
-      printf ("%s %8s %8s %10ld %s %s%s%s\n", 
-#endif
-              fle->attribs, fle->user, fle->group,
-              fle->size, buf, color, fle->file, COLOR_DEFAULT);
+      tempstr = gftp_gen_ls_string (fle, color, COLOR_DEFAULT);
+      printf ("%s\n", tempstr);
+      g_free (tempstr);
+
       delitem = templist;
     }
+
   gftp_end_transfer (request);
 
   if (delitem != NULL)
@@ -664,13 +694,15 @@ gftp_text_binary (gftp_request * request, char *command, gpointer *data)
 {
   if (!GFTP_IS_CONNECTED (gftp_text_remreq))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
 
-  gftp_set_data_type (gftp_text_remreq, GFTP_TYPE_BINARY);
-  gftp_set_data_type (gftp_text_locreq, GFTP_TYPE_BINARY);
+  gftp_set_request_option (gftp_text_remreq, "ascii_transfers", 
+                           GINT_TO_POINTER(0));
+  gftp_set_request_option (gftp_text_locreq, "ascii_transfers", 
+                           GINT_TO_POINTER(0));
   return (1);
 }
 
@@ -680,13 +712,15 @@ gftp_text_ascii (gftp_request * request, char *command, gpointer *data)
 {
   if (!GFTP_IS_CONNECTED (gftp_text_remreq))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
 
-  gftp_set_data_type (gftp_text_remreq, GFTP_TYPE_ASCII);
-  gftp_set_data_type (gftp_text_locreq, GFTP_TYPE_ASCII);
+  gftp_set_request_option (gftp_text_remreq, "ascii_transfers", 
+                           GINT_TO_POINTER(1));
+  gftp_set_request_option (gftp_text_locreq, "ascii_transfers", 
+                           GINT_TO_POINTER(1));
   return (1);
 }
 
@@ -699,18 +733,19 @@ gftp_text_mget_file (gftp_request * request, char *command, gpointer *data)
 
   if (!GFTP_IS_CONNECTED (gftp_text_remreq))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
 
   if (*command == '\0')
     {
-      gftp_text_log (gftp_logging_error, NULL, _("usage: mget <filespec>\n"));
+      gftp_text_log (gftp_logging_error, request, 
+                     _("usage: mget <filespec>\n"));
       return (1);
     }
 
-  transfer = g_malloc0 (sizeof (*transfer));
+  transfer = gftp_tdata_new ();
   transfer->fromreq = gftp_text_remreq;
   transfer->toreq = gftp_text_locreq;
   transfer->transfer_direction = GFTP_DIRECTION_DOWNLOAD;
@@ -772,18 +807,19 @@ gftp_text_mput_file (gftp_request * request, char *command, gpointer *data)
 
   if (!GFTP_IS_CONNECTED (gftp_text_remreq))
     {
-      gftp_text_log (gftp_logging_error, NULL,
+      gftp_text_log (gftp_logging_error, request,
                      _("Error: Not connected to a remote site\n"));
       return (1);
     }
 
   if (*command == '\0')
     {
-      gftp_text_log (gftp_logging_error, NULL, _("usage: mput <filespec>\n"));
+      gftp_text_log (gftp_logging_error, request, 
+                     _("usage: mput <filespec>\n"));
       return (1);
     }
 
-  transfer = g_malloc0 (sizeof (*transfer));
+  transfer = gftp_tdata_new ();
   transfer->fromreq = gftp_text_locreq;
   transfer->toreq = gftp_text_remreq;
   transfer->transfer_direction = GFTP_DIRECTION_UPLOAD;
@@ -839,12 +875,16 @@ gftp_text_mput_file (gftp_request * request, char *command, gpointer *data)
 int
 gftp_text_transfer_files (gftp_transfer * transfer)
 {
-  char *tempstr, buf[8192], *progress = "|/-\\";
+  int i, j, sw, tot, preserve_permissions;
+  char buf[8192], *progress = "|/-\\";
   struct timeval updatetime;
   long fromsize, total;
   gftp_file * curfle;
-  int i, j, sw, tot;
   ssize_t num_read;
+  mode_t mode;
+
+  gftp_lookup_request_option (transfer->fromreq, "preserve_permissions",
+                              &preserve_permissions);
 
   for (transfer->curfle = transfer->files;
        transfer->curfle != NULL;
@@ -860,13 +900,6 @@ gftp_text_transfer_files (gftp_transfer * transfer)
           continue;
         }
 
-      if (maxkbs > 0)
-       {
-          gftp_text_log (gftp_logging_misc, NULL, 
-                         _("File transfer will be throttled to %.2f KB/s\n"), 
-                         maxkbs);
-        }
-
       transfer->curtrans = curfle->startsize;
       fromsize = gftp_transfer_file (transfer->fromreq, curfle->file, -1,
                                      curfle->startsize, transfer->toreq, curfle->destfile, 
@@ -881,6 +914,7 @@ gftp_text_transfer_files (gftp_transfer * transfer)
   
       total = 0;
       i = 0;
+      num_read = -1;
       while (!cancel && (num_read = gftp_get_next_file_chunk (transfer->fromreq,
                                                         buf, sizeof (buf))) > 0)
         {
@@ -890,7 +924,7 @@ gftp_text_transfer_files (gftp_transfer * transfer)
             i = 0;
 
           total += num_read;
-          gftp_text_calc_kbs (transfer, num_read);
+          gftp_calc_kbs (transfer, num_read);
           if (transfer->lasttime.tv_sec - updatetime.tv_sec >= 1 || total >= fromsize)
             {
               sw = gftp_text_get_win_size () - 20;
@@ -908,40 +942,42 @@ gftp_text_transfer_files (gftp_transfer * transfer)
               memcpy (&updatetime, &transfer->lasttime, sizeof (updatetime));
             }
 
-          if (GFTP_GET_DATA_TYPE (transfer->fromreq) == GFTP_TYPE_ASCII)
-            tempstr = gftp_convert_ascii (buf, &num_read, 1);
-          else
-            tempstr = buf;
-
-          if (gftp_put_next_file_chunk (transfer->toreq, tempstr, num_read) < 0)
+          if (gftp_put_next_file_chunk (transfer->toreq, buf, num_read) < 0)
             {
               num_read = -1;
               break;
             }
-
-          /* We don't have to free tempstr for a download because new memory is
-             not allocated for it in that case */
-          if (GFTP_GET_DATA_TYPE (transfer->fromreq) == 
-              GFTP_TYPE_ASCII && !transfer->transfer_direction)
-            g_free (tempstr);
         }
       printf ("\n");
 
       if (num_read < 0)
         {
-          gftp_text_log (gftp_logging_misc, NULL, 
+          gftp_text_log (gftp_logging_misc, transfer->fromreq, 
                          _("Could not download %s\n"), curfle->file);
           gftp_disconnect (transfer->fromreq);
           gftp_disconnect (transfer->toreq);
         }
       else
         {
-          gftp_text_log (gftp_logging_misc, NULL, 
+          gftp_text_log (gftp_logging_misc, transfer->fromreq, 
                          _("Successfully transferred %s\n"), curfle->file);
           gftp_end_transfer (transfer->fromreq);
           gftp_end_transfer (transfer->toreq);
         }
 
+      if (!curfle->is_fd && preserve_permissions)
+        {
+          if (curfle->attribs)
+            {
+              mode = gftp_parse_attribs (curfle->attribs);
+              if (mode != 0)
+                gftp_chmod (transfer->toreq, curfle->destfile, mode);
+            } 
+
+          if (curfle->datetime != 0)
+            gftp_set_file_time (transfer->toreq, curfle->destfile,
+                                curfle->datetime);
+        }
     }
   return (1);
 }
@@ -1014,31 +1050,31 @@ gftp_text_help (gftp_request * request, char *command, gpointer *data)
 int
 gftp_text_set (gftp_request * request, char *command, gpointer *data)
 {
+  gftp_config_vars * cv, newcv;
   char *pos, *backpos;
+  GList * templist;
   int i;
 
   if (command == NULL || *command == '\0')
     {
-      for (i=0; config_file_vars[i].key != NULL; i++)
+      for (templist = gftp_options_list; 
+           templist != NULL; 
+           templist = templist->next)
         {
-          if (!(config_file_vars[i].ports_shown & GFTP_PORT_TEXT))
-            continue;
+          cv = templist->data;
 
-          switch (config_file_vars[i].type)
+          for (i=0; cv[i].key != NULL; i++)
             {
-              case CONFIG_CHARTEXT:
-                printf ("%s = %s\n", config_file_vars[i].key, 
-                        *(char **) config_file_vars[i].var);
-                break;
-              case CONFIG_INTTEXT:
-              case CONFIG_CHECKBOX:
-                printf ("%s = %d\n", config_file_vars[i].key, 
-                        *(int *) config_file_vars[i].var);
-                break;
-              case CONFIG_FLOATTEXT:
-                printf ("%s = %.2f\n", config_file_vars[i].key, 
-                        *(float *) config_file_vars[i].var);
-                break;
+              if (!(cv[i].ports_shown & GFTP_PORT_TEXT))
+                continue;
+
+              if (*cv[i].key == '\0' || 
+                  gftp_option_types[cv[i].otype].write_function == NULL)
+                continue;
+
+              printf ("%s = ", cv[i].key);
+              gftp_option_types[cv[i].otype].write_function (&cv[i], stdout, 0);
+              printf ("\n");
             }
         }
     }
@@ -1046,7 +1082,7 @@ gftp_text_set (gftp_request * request, char *command, gpointer *data)
     {
       if ((pos = strchr (command, '=')) == NULL)
         {
-          gftp_text_log (gftp_logging_error, NULL,
+          gftp_text_log (gftp_logging_error, request,
                          _("usage: set [variable = value]\n"));
           return (1);
         }
@@ -1058,48 +1094,31 @@ gftp_text_set (gftp_request * request, char *command, gpointer *data)
         *backpos = '\0';
       for (++pos; *pos == ' ' || *pos == '\t'; pos++);
 
-      for (i=0; config_file_vars[i].key != NULL; i++)
+      if ((cv = g_hash_table_lookup (gftp_global_options_htable, command)) == NULL)
         {
-          if (strcmp (config_file_vars[i].key, command) == 0)
-            break;
-           
-        }
-
-      if (config_file_vars[i].key == NULL)
-        {
-          gftp_text_log (gftp_logging_error, NULL,
+          gftp_text_log (gftp_logging_error, request,
                          _("Error: Variable %s is not a valid configuration variable.\n"), command);
           return (1);
         }
 
-      if (!(config_file_vars[i].ports_shown & GFTP_PORT_TEXT))
+      if (!(cv->ports_shown & GFTP_PORT_TEXT))
         {
-          gftp_text_log (gftp_logging_error, NULL,
+          gftp_text_log (gftp_logging_error, request,
                          _("Error: Variable %s is not available in the text port of gFTP\n"), command);
           return (1);
         }
 
-      configuration_changed = 1;
-      switch (config_file_vars[i].type)
+      if (gftp_option_types[cv->otype].read_function != NULL)
         {
-          case CONFIG_CHARTEXT:
-            if (*(char **) config_file_vars[i].var != NULL)
-              g_free (*(char **) config_file_vars[i].var);
-            *(char **) config_file_vars[i].var = g_strconcat (pos, NULL);
-            break;
-          case CONFIG_CHECKBOX:
-            *(int *) config_file_vars[i].var = *pos == '1' ? 1 : 0;
-            break;
-          case CONFIG_INTTEXT:
-            *(int *) config_file_vars[i].var = strtol (pos, NULL, 10);
-            break;
-          case CONFIG_FLOATTEXT:
-            *(float *) config_file_vars[i].var = strtod (pos, NULL);
-            break;
-          default:
-            gftp_text_log (gftp_logging_error, NULL,
-                           _("Error: You cannot change this variable\n"));
-            break;
+          memcpy (&newcv, cv, sizeof (newcv));
+          newcv.flags &= ~GFTP_CVARS_FLAGS_DYNMEM;
+
+          gftp_option_types[cv->otype].read_function (pos, &newcv, 1);
+
+          gftp_set_global_option (command, newcv.value);
+
+          if (newcv.flags & GFTP_CVARS_FLAGS_DYNMEM)
+            g_free (newcv.value);
         }
     }
 
@@ -1113,7 +1132,7 @@ gftp_text_clear (gftp_request * request, char *command, gpointer *data)
   if (strcasecmp (command, "cache") == 0)
     gftp_clear_cache_files ();
   else
-    gftp_text_log (gftp_logging_error, NULL, "Invalid argument\n");
+    gftp_text_log (gftp_logging_error, request, _("Invalid argument\n"));
   return (1);
 }
 
@@ -1124,7 +1143,7 @@ gftp_text_ask_question (const char *question, int echo, char *buf, size_t size)
   struct termios term, oldterm;
   sigset_t sig, sigsave;
   char *pos, *termname;
-  FILE *infd, *outfd;
+  FILE *infd;
 
   if (!echo)
     {
@@ -1138,10 +1157,9 @@ gftp_text_ask_question (const char *question, int echo, char *buf, size_t size)
         {
           
           gftp_text_log (gftp_logging_error, NULL, 
-                         "Cannot open controlling terminal %s\n", termname);
+                         _("Cannot open controlling terminal %s\n"), termname);
           return (NULL);
         }
-      outfd = infd;
 
       tcgetattr (0, &term);
       oldterm = term;
@@ -1149,12 +1167,9 @@ gftp_text_ask_question (const char *question, int echo, char *buf, size_t size)
       tcsetattr (fileno (infd), TCSAFLUSH, &term);
     }
   else
-    {
-      infd = stdin;
-      outfd = stdout;
-    }
+    infd = stdin;
 
-  fprintf (outfd, "%s%s%s: ", COLOR_BLUE, question, COLOR_DEFAULT);
+  printf ("%s%s%s: ", COLOR_BLUE, question, COLOR_DEFAULT);
 
   if (fgets (buf, size, infd) == NULL)
     return (NULL);
@@ -1162,9 +1177,9 @@ gftp_text_ask_question (const char *question, int echo, char *buf, size_t size)
 
   if (!echo)
     {
-      fprintf (outfd, "\n");
+      printf ("\n");
       tcsetattr (fileno (infd), TCSAFLUSH, &oldterm);
-      fclose (outfd);
+      fclose (infd);
       sigprocmask (SIG_SETMASK, &sigsave, NULL);
     }
 
@@ -1194,57 +1209,6 @@ gftp_text_get_win_size (void)
 }
 
 
-void
-gftp_text_calc_kbs (gftp_transfer * tdata, ssize_t num_read)
-{
-  unsigned long waitusecs;
-  double difftime, curkbs;
-  gftp_file * tempfle;
-  struct timeval tv;
-  unsigned long toadd;
-
-  gettimeofday (&tv, NULL);
-
-  tempfle = tdata->curfle->data;
-  tdata->trans_bytes += num_read;
-  tdata->curtrans += num_read;
-  tdata->stalled = 0;
-
-  difftime = (tv.tv_sec - tdata->starttime.tv_sec) + ((double) (tv.tv_usec - tdata->starttime.tv_usec) / 1000000.0);
-  if (difftime == 0)
-    tdata->kbs = (double) tdata->trans_bytes / 1024.0;
-  else
-    tdata->kbs = (double) tdata->trans_bytes / 1024.0 / difftime;
-
-  difftime = (tv.tv_sec - tdata->lasttime.tv_sec) + ((double) (tv.tv_usec - tdata->lasttime.tv_usec) / 1000000.0);
-
-  if (difftime <= 0)
-    curkbs = (double) (num_read / 1024.0);
-  else
-    curkbs = (double) (num_read / 1024.0 / difftime);
-
-  if (tdata->fromreq->maxkbs > 0 && 
-      curkbs > tdata->fromreq->maxkbs)
-    {
-      waitusecs = (double) num_read / 1024.0 / tdata->fromreq->maxkbs * 1000000.0 - difftime;
-
-      if (waitusecs > 0)
-        {
-          difftime += ((double) waitusecs / 1000000.0);
-          usleep (waitusecs);
-        }
-    }
-
-  /* I don't call gettimeofday (&tdata->lasttime) here because this will use
-     less system resources. This will be close enough for what we need */
-  difftime += tdata->lasttime.tv_usec / 1000000.0;
-  toadd = (long) difftime;
-  difftime -= toadd;
-  tdata->lasttime.tv_sec += toadd;
-  tdata->lasttime.tv_usec = difftime * 1000000.0;
-}
-
-
 void
 sig_child (int signo)
 {
@@ -1254,15 +1218,12 @@ sig_child (int signo)
 int
 gftp_text_set_show_subhelp (char *topic)
 {
-  int i;
+  gftp_config_vars * cv;
 
-  for (i=0; config_file_vars[i].key != NULL; i++)
+  if ((cv = g_hash_table_lookup (gftp_global_options_htable, topic)) != NULL)
     {
-      if (strcmp (topic, config_file_vars[i].key) == 0)
-        {
-          printf ("%s\n", config_file_vars[i].comment);
-          return (1);
-        }
+      printf ("%s\n", cv->comment);
+      return (1);
     }
 
   return (0);
@@ -1281,46 +1242,3 @@ gftp_text_clear_show_subhelp (char *topic)
   return (0);
 }
 
-
-#if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
-
-struct hostent *
-r_gethostbyname (const char *name, struct hostent *result_buf, int *h_errnop)
-{
-  struct hostent *hent;
-
-  if ((hent = gethostbyname (name)) == NULL)
-    {
-      if (h_errnop)
-        *h_errnop = h_errno;
-    }
-  else
-    {
-      *result_buf = *hent;
-      hent = result_buf;
-    }
-  return (hent);
-}
-
-#endif /* HAVE_GETADDRINFO */
-
-
-struct servent *
-r_getservbyname (const char *name, const char *proto,
-                 struct servent *result_buf, int *h_errnop)
-{
-  struct servent *sent;
-
-  if ((sent = getservbyname (name, proto)) == NULL)
-    {
-      if (h_errnop)
-        *h_errnop = h_errno;
-    }
-  else
-    {
-      *result_buf = *sent;
-      sent = result_buf;
-    }
-  return (sent);
-}
-
