@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  gftp-text.h - include file for the gftp text port                        */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -17,14 +17,14 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-/* $Id: gftp-text.h,v 1.3 2002/10/14 02:46:14 masneyb Exp $ */
+/* $Id: gftp-text.h,v 1.7 2003/07/25 20:39:35 masneyb Exp $ */
 
 #ifndef __GFTP_TEXT_H
 #define __GFTP_TEXT_H
 
 #include "../../lib/gftp.h"
 
-#ifdef HAVE_LIBREADLINE
+#if HAVE_LIBREADLINE
 #include <readline/readline.h>
 #include <readline/history.h>
 #endif
@@ -52,8 +52,8 @@ struct _gftp_text_methods
 
 /* gftp-text.h */
 void gftp_text_log				( gftp_logging_level level, 
-						  void *ptr, 
-						  const char *string, ... );
+						  gftp_request * request, 
+						  const char *string, ... ) __attribute__((format(printf, 3, 4)));
 int gftp_text_open				( gftp_request * request, 
 						  char *command, 
 						  gpointer *data );
@@ -117,8 +117,6 @@ char *gftp_text_ask_question 			( const char *question,
 						  char *buf,
 						  size_t size );
 int gftp_text_get_win_size 			( void );
-void gftp_text_calc_kbs 			( gftp_transfer * tdata, 
-						  ssize_t num_read );
 void sig_child 					( int signo );
 int gftp_text_set_show_subhelp			( char *topic );
 int gftp_text_clear_show_subhelp		( char *topic );
